/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>, Nils Behrens <psi@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

//only for debugging...
//#include "prelude.h"
//#include "sflsyst.h"


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <mpn/matrix.h>
#include <mpn/matrixpublish.h>
#include <time.h>
#include <string.h>



//Unix only
#if (defined (__UNIX__))
#include <unistd.h>
#endif
//end Unix only




char tmp_file[MPN_STRING_LEN];
char log_file[MPN_STRING_LEN];
int verbose;
int verbosefilter;
int runonce;
int beid;

int testpublish;
int testwait;
char testtext[MPN_STRING_LEN];

struct _friend {
  char name[MPN_STRING_LEN];
  char home[MPN_STRING_LEN];
  int poll;
  char cookie[MPN_STRING_LEN];
  int waitpoll;
};
typedef struct _friend friend;
typedef friend* friend_ptr;

struct _friend_list {
  int len;
  friend_ptr* list;
};
typedef struct _friend_list friend_list;
typedef friend_list* friend_list_ptr;

void log_string( const char* txt );


friend_list_ptr new_friend_list()
{
  friend_list_ptr fl;
  fl = calloc( sizeof( friend_list ), 1 );
  return fl;
}

void free_friend_list( friend_list_ptr fl )
{
  int i;
  if( fl->list ) {
    for( i=0; i<fl->len; i++ ) {
      free( fl->list[i] );
    }
    free( fl->list );
  }
  free( fl );
}

int identity_check()
{
  mpn_prefs_ptr prefsdoc;
  int r;

  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    printf( "identity_check: error opening prefs\n" );
    return 0;
  }

  if( beid==0 ) {
    if( mpn_prefs_get_identity_count( prefsdoc ) == 0 ) {
      mpn_prefs_close( prefsdoc );
      if( verbose ) printf( "no identity defined, not fetching news\n" );
      return 0;
    }
  } else {
    if( mpn_prefs_get_identity_count( prefsdoc ) < beid ) {
      mpn_prefs_close( prefsdoc );
      if( verbose ) printf( "selected identity does not exist, not fetching news\n" );
      return 0;
    }
  }      

  mpn_prefs_close( prefsdoc );
  return 1;
}

friend_list_ptr update_friend_list( friend_list_ptr fl )
{
  mpn_prefs_ptr prefsdoc;
  mpn_friend_ptr mpnfriend;
  mpn_person_ptr person;
  friend_list_ptr newfl;
  friend_ptr newfriend;
  int r, i, j;

  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    printf( "update_friend_list: error opening prefs\n" );
    return fl;
  }

  newfl = new_friend_list();

  for( i=0; i<mpn_prefs_get_friend_count( prefsdoc ); i++ ) {

    mpnfriend = mpn_prefs_get_friend( prefsdoc, i );
    newfriend = calloc( sizeof( friend ), 1 );

    person = mpn_friend_get_person( mpnfriend );
    strcpy( newfriend->name, mpn_person_get_name( person ) );
    strcpy( newfriend->home, mpn_person_get_home( person ) );
    newfriend->poll = mpn_friend_get_poll( mpnfriend );
    strcpy( newfriend->cookie, mpn_friend_get_cookie( mpnfriend ) );

    newfriend->waitpoll = 1;
    for( j=0; j<fl->len; j++ ) {
      if( strcmp( fl->list[j]->home, newfriend->home )==0 ) {
	newfriend->waitpoll = fl->list[j]->waitpoll + 1;
      }
    }

    newfl->list = realloc( newfl->list, sizeof( friend_ptr ) * (newfl->len+1) );
    newfl->list[newfl->len] = newfriend;
    newfl->len++;
  }

  mpn_prefs_close( prefsdoc );
  return newfl;
}

void add_newsdoc( mpn_news_ptr ownnews, mpn_news_ptr newnewsdoc, 
		  mpn_identity_ptr identity, char* logbuf )
{
  mpn_newsitem_ptr oldnews;
  mpn_newsitem_ptr newnews;
  mpn_net_ptr oldnet;
  mpn_net_ptr newnet;
  int i, j, brandnew, recommend, ttl, dontrecommend;
  char buf[MPN_STRING_LEN];

  for( i=0; i<mpn_news_get_len( newnewsdoc ); i++ ) {

    newnews = mpn_news_get_item( newnewsdoc, i );
    brandnew = 1;
    recommend = 1;

    // Is it already here?
    for( j=0; j<mpn_news_get_len( ownnews ); j++ ) {
      oldnews = mpn_news_get_item( ownnews, j );
      if( mpn_newsitem_equal( newnews, oldnews ) ) {

	oldnet = mpn_newsitem_get_net( oldnews );
	newnet = mpn_newsitem_get_net( newnews );
	
	if( mpn_net_get_weight(newnet) > mpn_net_get_weight(oldnet) ) {
	  sprintf( buf,"%s, already here -> updating weight %.3f to %.3f\n", 
		   logbuf, mpn_net_get_weight(oldnet), 
		   mpn_net_get_weight(newnet) );
	  log_string( buf );
	  if( verbose ) printf( "message already here, updating weight\n" );
	  mpn_net_set_weight( ownnews, oldnet, mpn_net_get_weight(newnet) );
	} else {
	  strcat( logbuf, ", already here -> dropping\n" ); 
	  log_string( logbuf );
	  if( verbose ) printf( "message already here, dropping new one\n" );
	  recommend = 0;
	}

	brandnew = 0;
	break;
      }
    }
    
    if( brandnew ) {
      // Finally we add the message to our own newslist
      sprintf( buf, "%s -> adding\n", logbuf ); 
      log_string( buf );
      if( verbose ) printf( "adding new message\n" );
      mpn_news_add_item( ownnews, newnews );
    }

    if( recommend ) {
      newnet = mpn_newsitem_get_net( newnews );
      ttl = mpn_net_get_ttl( newnet );
      dontrecommend = mpn_net_get_dontrecommend( newnet );

      if( dontrecommend ) {
	sprintf( buf, "%s, dont_recommend set\n", logbuf );
	log_string( buf );
	if( verbose ) printf( "dont_recommend set\n" );

      } else if( ttl>0 ) {
	sprintf( buf, "%s -> recommending\n", logbuf ); 
	log_string( buf );
	if( verbose ) printf( "recommending message\n" );
	mpn_recommend_newsitem( identity, newnewsdoc, newnews );

      } else {
	sprintf( buf, "%s, ttl==0 -> not recommending\n", logbuf ); 
	log_string( buf );
	if( verbose ) printf( "ttl==0, not recommending message\n" );
      }
    }

  }
}

int add_newslist( friend_ptr fr, mpn_news_ptr importnews )
{
  int r, i;
  mpn_news_ptr ownnews, newnewsdoc, odoc, ffdoc, gfdoc;
  mpn_newsitem_ptr newnews;
  mpn_sig_ptr sig;
  mpn_date_ptr mpndate;
  mpn_net_ptr net;
  mpn_filter_ptr globalfilter, friendfilter, deffriendfilter, defbotfilter;
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr identity;
  mpn_friend_ptr friend;
  mpn_person_ptr person;
  int ttl, hop, isbot;
  char buf[MPN_STRING_LEN];

  r = mpn_news_open_private( MPN_OPEN_READWRITE, &ownnews );
  if( r!=MPN_OK ) {
    fprintf( stderr, "error opening private newslist\n" );
    return MPN_PERM_ERR;
  }

  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "error opening private prefs\n" );
    mpn_news_close( ownnews );
    return MPN_PERM_ERR;
  }

  // Select Identity
  if( beid==0 ) {
    identity = mpn_prefs_get_identity( prefsdoc, 
			   mpn_prefs_get_identity_default( prefsdoc ) );
  } else {
    if( beid-1>=mpn_prefs_get_identity_count( prefsdoc ) ) {
      fprintf( stderr, "selected identity does not exist\n" );
      mpn_news_close( ownnews );
      mpn_prefs_close( prefsdoc );
      return MPN_PERM_ERR;
    }
    identity = mpn_prefs_get_identity( prefsdoc, beid-1 );
  }
  if( !identity ) {
    mpn_news_close( ownnews );
    mpn_prefs_close( prefsdoc );
    if( verbose ) printf( "no identity defined in prefs.xml\n" );
    return MPN_PERM_ERR;
  }

  // Get Global Filter
  globalfilter = mpn_prefs_get_globalfilter( prefsdoc );
  deffriendfilter = mpn_prefs_get_deffriendfilter( prefsdoc );
  defbotfilter = mpn_prefs_get_defbotfilter( prefsdoc );

  // Get Friendfilter
  friendfilter = NULL; 
  isbot = 0;
  for( i=0; i<mpn_prefs_get_friend_count( prefsdoc ); i++ ) {
    friend = mpn_prefs_get_friend( prefsdoc, i );
    person = mpn_friend_get_person( friend );
    if( ( strcmp(fr->home,mpn_person_get_home(person)) == 0 ) 
	&& ( strcmp(fr->name,mpn_person_get_name(person)) == 0 ) ) {
      friendfilter = mpn_friend_get_filter( friend );
      isbot = mpn_friend_is_bot( friend );
      break;
    }
  }

  for( i=0; i<mpn_news_get_len( importnews ); i++ ) {
    
    // Check out the next item
    newnews = mpn_news_get_item( importnews, i );

    sig = mpn_newsitem_get_sig( newnews );
    sprintf( buf, "incoming from %s: %s", fr->name, mpn_sig_get_id( sig ) );

    // Is it expired?
    if( sig ) {
      mpndate = mpn_sig_get_expire( sig );
      if( mpndate ) {
	if( time( NULL )>mpn_date_get_sec( mpndate ) ) {
	  strcat( buf, ", expired -> dropping\n" );
	  log_string( buf );
	  if( verbose ) printf( "message expired\n" ); 
	  continue;
	}
      }
    } else {
      if( verbose ) printf( "message has no sig section\n" );
      continue;
    }

    // Process .NET
    net = mpn_newsitem_get_net( newnews );
    if( net ) {
      // TTL
      ttl = mpn_net_get_ttl( net );
      if( ttl<=0 ) {
	strcat( buf, ", ttl<0 -> dropping\n" );
	log_string( buf );
	if( verbose ) printf( "message ttl<0, died\n" );
	continue;
      }
      ttl--;
      mpn_net_set_ttl( ownnews, net, ttl );

      // HOP count
      hop = mpn_net_get_hop( net )+1;
      mpn_net_set_hop( ownnews, net, hop );

      // Remove localfile entry
      mpn_net_set_localfile( ownnews, net, "" );
    } else {
      if( verbose ) printf( "message has no net section\n" );
      continue;
    }

    mpn_news_open_empty( &odoc );
    mpn_news_add_item( odoc, newnews );
    newnewsdoc = odoc;

    // Friend Processing
    if( friendfilter ) {
      if( verbose ) printf( "processing with friend filter\n" );
      
      r = mpn_news_filter( newnewsdoc, friendfilter, &ffdoc, verbosefilter );
      if( r==MPN_OK ) {
	newnewsdoc = ffdoc;
      } else {
	fprintf( stderr, "error in friendfilter\n" );
	ffdoc = NULL;
      }

    } else if( isbot ) {
      if( defbotfilter ) {
	if( verbose ) printf( "processing with default bot filter\n" );
      
	r = mpn_news_filter( newnewsdoc, defbotfilter, &ffdoc, verbosefilter );
	if( r==MPN_OK ) {
	  newnewsdoc = ffdoc;
	} else {
	  fprintf( stderr, "error in default botfilter\n" );
	  ffdoc = NULL;
	}

      } else {
	fprintf( stderr, "warning: no filter defined for this bot\n" );
	ffdoc = NULL;
      }

    } else if( !isbot ) {
      if( deffriendfilter ) {
	if( verbose ) printf( "processing with default friend filter\n" );
      
	r = mpn_news_filter(newnewsdoc,deffriendfilter,&ffdoc,verbosefilter );
	if( r==MPN_OK ) {
	  newnewsdoc = ffdoc;
	} else {
	  fprintf( stderr, "error in default friendfilter\n" );
	  ffdoc = NULL;
	}

      } else {
	fprintf( stderr, "warning: no filter defined for this friend\n" );
	ffdoc = NULL;
      }
    }
    
    if( mpn_news_get_len( newnewsdoc )>0 ) {
      
      // Apply Global Filter
      if( globalfilter ) {
	if( verbose ) printf( "processing with global filter\n" );
	
	r = mpn_news_filter( newnewsdoc, globalfilter, &gfdoc,verbosefilter);
	if( r==MPN_OK ) {
	  newnewsdoc = gfdoc;
	} else {
	  fprintf( stderr, "error in globalfilter\n" );
	  gfdoc = NULL;
	}
      } else {
	gfdoc = NULL;
      }
      
      if( mpn_news_get_len( newnewsdoc )>0 ) {
	
	add_newsdoc( ownnews, newnewsdoc, identity, buf );
	
      } else {
	strcat( buf, ", did not pass friendfilter -> dropping\n" );
	log_string( buf );
	if( verbose ) printf( "message did not pass the filter\n" );
      }
      
      if( gfdoc ) mpn_news_close( gfdoc );
    } else {
      strcat( buf, ", did not pass global filter -> dropping\n" );
      log_string( buf );
      if( verbose ) printf( "message did not pass the filter\n" );
    }
    
    if( ffdoc ) mpn_news_close( ffdoc );
    mpn_news_close( odoc );
    
  }
  
  mpn_prefs_close( prefsdoc );
  mpn_news_close( ownnews );
  return MPN_OK;
}

void save_cookie( friend_ptr fr, mpn_news_ptr importnews )
{
  const char* cookie;
  mpn_prefs_ptr prefsdoc;
  mpn_friend_ptr mpnfriend;
  mpn_person_ptr person;
  int r, i;

  cookie = mpn_news_get_cookie( importnews );
  if( cookie[0] ) {
    
    r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
    if( r!=MPN_OK ) return;

    for( i=0; i<mpn_prefs_get_friend_count( prefsdoc ); i++ ) {
      mpnfriend = mpn_prefs_get_friend( prefsdoc, i );
      person = mpn_friend_get_person( mpnfriend );
      if( strcmp( mpn_person_get_home( person ), fr->home )==0 ) {
	if( strcmp( mpn_person_get_name( person ), fr->name ) == 0 ) {
	  mpn_friend_set_cookie( prefsdoc, mpnfriend, cookie );
	  break;
	}
      }
    }

    mpn_prefs_close( prefsdoc );
  }
}

void fetchnews_one( friend_ptr fr )
{
  int r, r2;
  mpn_news_ptr importnews;

  r = mpn_fetch_newslist( fr->home, fr->cookie, tmp_file );
  if( r!=MPN_OK ) {
    printf( "error fetching newslist %s\n", fr->home );
    return;
  }

  if( verbose ) printf( "fetching list OK\n" );

  r = mpn_news_open_foreign( tmp_file, MPN_OPEN_READONLY, &importnews, 
			     "matrix-public-news" );
  if( r==MPN_OK ) {
    if( mpn_news_get_len( importnews ) > 0 ) {
    
      if( verbose ) printf( "list has %d messages\n", 
			    mpn_news_get_len( importnews ) );
      r2 = add_newslist( fr, importnews );
      if( r2==MPN_OK ) 
	save_cookie( fr, importnews );

    } else {
      if( verbose ) printf( "fetched list is empty\n" );
    }
    
    mpn_news_close( importnews );

  } else {
    fprintf( stderr, "error opening fetched newslist\n" );
  }
  unlink( tmp_file );
}

friend_list_ptr fetchnews_all( friend_list_ptr fl )
{
  friend_list_ptr newfl;
  int i;

  newfl = update_friend_list( fl );

  if( verbose ) printf( "checking friendlist (%d friends)\n", newfl->len );

  for( i=0; i<newfl->len; i++ ) {
    if( (newfl->list[i]->waitpoll >= newfl->list[i]->poll)||runonce ) {
      if( verbose ) printf( "fetching list from %s (%s)\n", 
			    newfl->list[i]->name, newfl->list[i]->home );
      fetchnews_one( newfl->list[i] );
      newfl->list[i]->waitpoll = 0;

    } else {
      if( verbose ) printf( "waiting %d/%d min for %s\n",
			    newfl->list[i]->waitpoll, newfl->list[i]->poll,
			    newfl->list[i]->name );
    }
  }

  return newfl;
}

void clean_expired()
{
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr id;
  mpn_news_ptr newsdoc;
  mpn_newsitem_ptr newsitem;
  mpn_net_ptr net;
  mpn_sig_ptr sig;
  mpn_date_ptr date;
  int reserve;
  int r, i;
  long expire;
  char filename[MPN_STRING_LEN];
  char buf[MPN_STRING_LEN];

  if( verbose ) {
    printf( "looking for expired items\n" );
  }

  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "clean_expired: error opening prefs.\n" );
    return;
  }

  if( beid==0 ) {
    id = mpn_prefs_get_identity( prefsdoc, 
				 mpn_prefs_get_identity_default( prefsdoc ) );
  } else {
    id = mpn_prefs_get_identity( prefsdoc, beid-1 );
  }
  if( !id ) {
    mpn_prefs_close( prefsdoc );
    if( verbose ) printf( "no identity defined in prefs.xml\n" );
    return;
  }

  reserve = mpn_identity_get_reserve( id );
  mpn_prefs_close( prefsdoc );

  r = mpn_news_open_private( MPN_OPEN_READWRITE, &newsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "clean_expired: error opening private news.\n" );
    return;
  }

  for( i=0; i<mpn_news_get_len( newsdoc ); i++ ) {

    newsitem = mpn_news_get_item( newsdoc, i );
    net = mpn_newsitem_get_net( newsitem );

    if( !mpn_net_get_keep( net ) ) {

      sig = mpn_newsitem_get_sig( newsitem );
      date = mpn_sig_get_expire( sig );
      
      if( date ) {
	expire = mpn_date_get_sec( date );
	if( expire+reserve*60*60 < time( NULL ) ) {
	  
	  sprintf(buf,"expired: removing newsitem %s\n",mpn_sig_get_id(sig));
	  log_string( buf );
	  if( verbose ) {
	    printf( buf ); 
	  }
	  
	  if( strcmp( "", mpn_net_get_localfile( net ) )!=0 ) {
	    sprintf( filename, "%s/%s", mpn_get_contentdir(), 
		     mpn_net_get_localfile( net ) );
	    sprintf( buf, "expired: removing file %s\n", filename );
	    log_string( buf );
	    if( verbose ) {
	      printf( buf );
	    }
	    unlink( filename );
	  }
	  
	  mpn_news_del_item( newsdoc, newsitem );
	  i--;
	}
      }
    }    
  }
  
  mpn_news_close( newsdoc );
}

void publish_test()
{
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr id;
  mpn_person_ptr person;
  mpn_news_ptr newsdoc;
  mpn_newsitem_ptr newsitem;
  mpn_sig_ptr sig;
  char buf[MPN_STRING_LEN];
  char buf2[MPN_STRING_LEN];
  char idbuf[MPN_STRING_LEN];
  int r;
  long expire;
  
  if( testpublish==0 ) return;

  testwait--;
  if( testwait>0 ) return;
  testwait = testpublish;

  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "publish_test: error opening prefs-file\n" );
    return;
  }
  if( beid==0 ) {
    id = mpn_prefs_get_identity( prefsdoc, 
				 mpn_prefs_get_identity_default( prefsdoc ) );
  } else {
    id = mpn_prefs_get_identity( prefsdoc, beid-1 );
  }
  if( !id ) {
    mpn_prefs_close( prefsdoc );
    if( verbose ) printf( "no identity defined in prefs.xml\n" );
    return;
  }

  person = mpn_identity_get_person( id );
  sprintf( buf, "%s-%s", mpn_person_get_name( person ), testtext );
  expire = time( NULL ) + 60*60*5;

  r = mpn_news_compose_default( id, &newsdoc, buf, "", expire, testtext, "" );

  if( r!=MPN_OK ) {
    mpn_prefs_close( prefsdoc );
    fprintf( stderr, "publish_test: error composing msg\n" );
    return;
  }

  newsitem = mpn_news_get_item( newsdoc, 0 );
  sig = mpn_newsitem_get_sig( newsitem );
  strcpy( idbuf, mpn_sig_get_id( sig ) );

  r = mpn_recommend_newsitem( id, newsdoc, newsitem );
  //r = mpn_publish_newsitem_file( id, newsdoc, newsitem, NULL );

  if( r!=MPN_OK ) {
    fprintf( stderr, "publish_test: error publishing msg\n" );
  }

  sprintf( buf2, "test: msg \"%s\" published\n", idbuf );
  log_string( buf2 );
  if( verbose ) {
    printf( "published test msg \"%s\"\n", idbuf );
  }

  mpn_news_close( newsdoc );
  mpn_prefs_close( prefsdoc );
}

int parse_cmdline_options( int argc, char **argv )
{
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr identity;
  mpn_person_ptr person;
  int i, r;
  
  runonce = 0;
  verbose = 0;
  verbosefilter = 0;
  beid = 0;
  testpublish = 0;
  testwait = 0;

  for( i=1; i<argc; i++ ) {
    
    if( (strcmp( "-v", argv[i] )==0)||(strcmp( "--verbose", argv[i] )==0) ) {
      if( !verbose ) verbose = 1;
      else verbosefilter = 1;
      
    } else if( (strcmp( "-o", argv[i] )==0)||(strcmp( "--once", argv[i])==0) ){
      runonce = 1;
      
    } else if( (strcmp("-h", argv[i])==0)||(strcmp("--help",argv[i])==0)) {
      return 0;
      
    } else if( (strcmp("-i", argv[i])==0)||(strcmp("--identity",argv[i])==0)) {
      if( argc>i+1 ) {
	beid = atoi( argv[i+1] );
	i++;
      } else {
	printf( "Number must follow %s\n", argv[i] );
	return 0;
      }
      
    } else if( (strcmp("-t", argv[i])==0)||(strcmp("--test",argv[i])==0)) {
      if( argc>i+2 ) {
	testpublish = atoi( argv[i+1] );
	strcpy( testtext, argv[i+2] );
	i+=2;
	
      } else {
	printf( "Number and text must follow %s\n", argv[i] );
	return 0;
      }
      
    } else {
      printf( "Invalid option -- \"%s\"\n", argv[i] );
      return 0;
    }
  }

  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "error opening private prefs\n" );
    return 0;
  }

  if( beid==0 ) {
    identity = mpn_prefs_get_identity( prefsdoc, 
		   mpn_prefs_get_identity_default( prefsdoc ) );
    if( !identity ) {
      printf( "no identity defined in prefs.xml\n" );
    } else {
      if( verbose ) {
	person = mpn_identity_get_person( identity );
	printf("Using default identity: \"%s\"\n",mpn_person_get_name( person ));
      }
    }
  } else {
    if( (beid<=0)||(beid-1>=mpn_prefs_get_identity_count( prefsdoc )) ) {
      printf( "selected identity does not exist\n" );
      mpn_prefs_close( prefsdoc );
      //return 1;
    } else {
      identity = mpn_prefs_get_identity( prefsdoc, beid-1 );
      person = mpn_identity_get_person( identity );
      if( verbose ) {
	printf("Using identity #%d: \"%s\"\n", 
	       beid, mpn_person_get_name( person ));
      }
    }
  }

  log_string( "started.\n" );

  mpn_prefs_close( prefsdoc );
  return 1;
}

void log_string( const char* txt ) 
{
  FILE* file;
  time_t t;
  char buf[MPN_STRING_LEN];

  file = fopen( log_file, "a" );
  if( file ) {
    t = time( NULL );
    strcpy( buf, ctime( &t ) );
    buf[ strlen( buf )-1 ] = 0;
    fprintf( file, "[%s] - %s", buf, txt );
    fclose( file );
  }
}

void print_usage()
{
  printf( "Usage: fetchnews [options]\n" );
  printf( "  Options are as follows:\n" );
  printf( "  -h, --help           show this help\n" );
  printf( "  -o, --once           check only once and exit\n" );
  printf( "  -v, --verbose        print debugging information,\n" );
  printf( "                       -v -v to debug stylesheet processing\n" );
  printf( "  -i n, --identity n   use identity #n, otherwise use default\n" );
  printf( "  -t n s, --test n s   publish every n minutes a test message\n" );
  printf( "                       with comment s\n" );
}

void beta_test()
{
 
  printf("Betatest Version des MATRIX Projekts\n");
  printf("====================================\n");
  printf("Sollten Sie nicht der unten angegebene\n");
  printf("User sein schreiben Sie bitte an:\n");
  printf("==>matrix-beta@wearlab.de.\n\n");
  printf("Wolfgang Runte\n");
  printf("Email:Woru@tzi.de\n");

  printf("Diese Version luft ab am: Sun Jul 21 19:09:37 UTC 2002\n");
  if(time(NULL)>1027278615){
  	printf("Betaversion abgelaufen.\n");
	exit(23);
  }
}

int main( int argc, char **argv )
{
  friend_list_ptr fl;
  friend_list_ptr newfl;
  int running;
  
  mpn_init();
  sprintf( tmp_file, "%s/tmp_news.xml", mpn_get_workdir() );
  sprintf( log_file, "%s/fetchnews.log", mpn_get_workdir() );
  
  if( !parse_cmdline_options( argc, argv ) ) {
    print_usage();
    exit( 1 );
  }

  //Use for Betatesting
  //beta_test();
  
  fl = new_friend_list();

  running = 1;
  while( running ) {

    if( identity_check() ) {
      
      clean_expired();
      publish_test();

      newfl = fetchnews_all( fl );
      if( newfl!=fl ) {
	free_friend_list( fl );
	fl = newfl;
      }
    }

    if( runonce ) running = 0;
    else mfl_sleep( 60000 );
    if( verbose ) printf( "\n" );

  }

  //if( verbose ) printf( "exiting gracefully\n" );

  free_friend_list( fl );
  mpn_cleanup();
  return 0;
}








