/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

/* Parts taken from
   vorbisfile_example.c (C) XIPHOPHORUS Company http://www.xiph.org/
   encoder_axample.c (C) XIPHOPHORUS Company http://www.xiph.org
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include "audio.h"
#include "support.h"
#include "reporter.h"
#include <vorbis/vorbisenc.h>
#include <vorbis/vorbisfile.h>

int dsp_fd;
int srate;

// Monitor Thread
int monitor_running;
pthread_t monitor_thread;

// Record Thread
int record_running;
pthread_t record_thread;

// Play Thread
int play_running;
pthread_t play_thread;

// Stop a Thread
int run_thread;

// Volume Bar
GtkWidget* vol_bar;
float max_volume;
gint timeout;

int play_timer;
int finished_playing;

#define MONITOR_BUF_SIZE (4096*4)
#define REC_BUF_SIZE (22050*120*2)
#define MP3_BUF_SIZE 8192
#define PLAY_BUF_SIZE 8192

#define OPEN_PLAY 0
#define OPEN_REC  1

signed short compressor[0x7FFF];

int rec_len;
signed short *rec_buf;

int save_ogg_file( char* filename, char* rec_buf, int rec_len );


void init_compressor()
{
  int i;
  double x, y;

  for( i=0; i<0x7FFF; i++ ) {
    x = (double)i/(double)0x7FFF;
    y = (1.0-(1.0/(x+1.0)))*2.0*32768.0;
    compressor[i] = (signed short)y;
    //if( i%1024==0 ) printf( "%d\n", compressor[i] );
    //if( compressor[i]>0x7FFF ) compressor[i]=0x7FFF;
    //if( compressor[i]<0 ) compressor[i]=0;
  }
  //printf( "%d\n", compressor[i-1] );
}

void init_audio()
{
  dsp_fd = -1;
  monitor_running = 0;
  record_running = 0;
  play_running = 0;
  max_volume = 0.0f;
  init_compressor();
}

void cleanup_audio()
{
}

int open_dsp( int mode )
{
  int channels = 1; // 1=2 chns, 0=1 chn
  int format = AFMT_S16_LE;
  int rate = 44100;

  //if ( (dsp_fd = open("/dev/dsp",O_RDWR)) == -1 ) {

  if( mode == OPEN_PLAY ) {
    if ( (dsp_fd = open("/dev/dsp",O_WRONLY)) == -1 ) {
      perror("open /dev/dsp");
      dsp_fd = -1;
      return -1;
    }
  } else {
    if ( (dsp_fd = open("/dev/dsp",O_RDONLY)) == -1 ) {
      perror("open /dev/dsp");
      dsp_fd = -1;
      return -1;
    }
  }

  /*
  if ( ioctl(dsp_fd, SNDCTL_DSP_SETDUPLEX, 0 ) == -1 ) {
    perror("set duplex");
    return errno;
  }
  */
  /*
  if ( ioctl(m_fd, SNDCTL_DSP_SETFRAGMENT, &setting) == -1 ) {
    perror("ioctl set fragment");
  }
  */
  if ( ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &format) == -1 ) {
    perror("ioctl format");
    return errno;
  }
  if ( ioctl(dsp_fd, SNDCTL_DSP_STEREO, &channels) == -1 ) {
    perror("ioctl stereo");
    return errno;
  }
  if ( ioctl(dsp_fd, SNDCTL_DSP_SPEED, &rate) == -1 ) {
    perror("ioctl sample rate");
    return errno;
  }

  srate = rate;
  return 0;
}

int close_dsp()
{
  if( dsp_fd != -1 ) 
    close( dsp_fd );
  return 0;
}

gint update_volume_bar_callback( gpointer data )
{
  gtk_progress_bar_update( GTK_PROGRESS_BAR(vol_bar), max_volume );
  return 1;
}

// MONITOR MONITOR MONITOR
void* monitor_loop( void* args ) 
{
  int in, i, max;
  //int out;
  signed short rec_buf[MONITOR_BUF_SIZE/2];
  float m;

  while( run_thread ) {

    in = read( dsp_fd, rec_buf, MONITOR_BUF_SIZE );
    if( in==-1 )
      continue;

    max = 0;
    for( i=0; i<in/2; i++ ) {

      if( rec_buf[i]>=0 ) rec_buf[i] = compressor[ rec_buf[i] ];
      else rec_buf[i] = -compressor[ -rec_buf[i] ];

      if( rec_buf[i]>max ) max=rec_buf[i];
      else if( -rec_buf[i]>max ) max=rec_buf[i];
    }
    m = ((float)max)/32768.0f;
    if( m>1.0f ) m=1.0f;
    else if( m<0.0f ) m=0.0f;
    max_volume = m;
    
    //out = write( dsp_fd, rec_buf, MONITOR_BUF_SIZE );

  }
  
  max_volume = 0.0f;
  return NULL;
}

void start_monitor()
{
  int r;

  monitor_running = 1;
  if( record_running ) stop_record();
  if( play_running ) stop_play();

  vol_bar = lookup_widget( composer, "volume_bar" );
  max_volume = 0.0f;
  run_thread = 1;

  r = open_dsp( OPEN_REC );
  if( r==0 ) {
    timeout = gtk_timeout_add( 50, update_volume_bar_callback, NULL );
    pthread_create( &monitor_thread, NULL, monitor_loop, NULL ); 

    status_text( "Monitoring ..." );

  } else {
    status_text( "Error opening sound device /dev/dsp" );
    //togglebutton = lookup_widget( composer, "monitor_button" );
    //gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(togglebutton), 0 );
  }
}

void stop_monitor()
{
  GtkWidget* togglebutton;

  run_thread = 0;
  pthread_join( monitor_thread, NULL );
  close_dsp();
  gtk_timeout_remove( timeout );

  togglebutton = lookup_widget( composer, "monitor_button" );
  if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(togglebutton) ) ) {
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(togglebutton), 0 );
  }
  monitor_running = 0;

  status_text( "" );
}

// RECORD RECORD RECORD
void* record_loop( void* args ) 
{
  int inp, i, max, add, y, z; //, rec_len;
  //signed short *rec_buf;
  float m;
  signed short first_buf[MONITOR_BUF_SIZE/2];

  rec_buf = malloc( REC_BUF_SIZE );
  if( rec_buf == 0 ) printf( "no mem\n" );

  rec_len = 0;

  while( run_thread && (rec_len+4096<REC_BUF_SIZE ) ) {

    //inp = read( dsp_fd, &(rec_buf[rec_len/2]), MONITOR_BUF_SIZE );
    inp = read( dsp_fd, first_buf, MONITOR_BUF_SIZE );

    //printf( "read: %d, pos: %d\n", inp, rec_len );

    if( inp==-1 ) {
	printf( "inp==-1\n" );
      continue;
    }

    if( srate==48000 ) {
	for( i=0, z=0, y=0; i<inp/2; i+=4, y++ ) {
	    if( y%13!=0 ) {
		rec_buf[rec_len/2+z] = first_buf[i];
		z++;
	    }
	}
	add = z*2;
    } else {
	for( i=0; i<inp/2; i+=4 ) {
	    rec_buf[rec_len/2+i/4] = first_buf[i];
	}
	add=inp/4;
    }

    max = 0;
    for( i=rec_len/2; i<(rec_len+add)/2; i++ ) {

      if( rec_buf[i]>=0 ) rec_buf[i] = compressor[ rec_buf[i] ];
      else rec_buf[i] = -compressor[ -rec_buf[i] ];

      if( rec_buf[i]>max ) max=rec_buf[i];
      else if( -rec_buf[i]>max ) max=rec_buf[i];
    }
    m = ((float)max)/32768.0f;
    if( m>1.0f ) m=1.0f;
    else if( m<0.0f ) m=0.0f;
    max_volume = m;

    rec_len += add;
  }

  max_volume = 0.0f;

  return NULL;
}

void start_record()
{
  int r;

  record_running = 1;
  if( monitor_running ) stop_monitor();
  if( play_running ) stop_play();

  vol_bar = lookup_widget( composer, "volume_bar" );
  max_volume = 0.0f;
  run_thread = 1;

  r = open_dsp( OPEN_REC );
  if( r==0 ) {
    timeout = gtk_timeout_add( 50, update_volume_bar_callback, NULL );
    pthread_create( &record_thread, NULL, record_loop, NULL ); 

    status_text( "Recording ..." );

  } else {
    status_text( "Error opening sound device /dev/dsp" );
  }

}

void stop_record()
{
  GtkWidget* togglebutton;
  GtkWidget* fileentry;

  run_thread = 0;
  pthread_join( record_thread, NULL );
  close_dsp();
  gtk_timeout_remove( timeout );

  status_text( "Encoding ..." );
  status_activity_on();

  save_ogg_file( TEMP_OGG_FILE, rec_buf, rec_len );
  free( rec_buf );

  togglebutton = lookup_widget( composer, "rec_button" );
  if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(togglebutton) ) ) {
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(togglebutton), 0 );
  }
  record_running = 0;

  // Set OGG File Name in GUI
  fileentry = lookup_widget( composer, "fileentry" );
  gtk_entry_set_text( fileentry, TEMP_OGG_FILE );

  status_activity_off();
  status_text( "" );
}

// PLAY PLAY PLAY
void* play_loop( void* args ) 
{
  int out, i, max;
  signed short play_buf[PLAY_BUF_SIZE/2];
  signed short scnd_buf[PLAY_BUF_SIZE*2];
  float m;
  FILE* oggfile;
  OggVorbis_File vf;
  int current_section=0;
  GtkWidget *fileentry;

  fileentry = lookup_widget( composer, "fileentry" );
  
  oggfile = fopen( (char*)(gtk_entry_get_text( fileentry ) ), "r" ); 

  if( !oggfile ) {
    printf( "OGG File not existant\n" );
    return NULL; 
  }

  if(ov_open(oggfile, &vf, NULL, 0) < 0) {
    fprintf(stderr,"File ist not an OGG File.\n");
    return NULL;
  }
  
  while( run_thread ) {

    out = ov_read(&vf,play_buf,PLAY_BUF_SIZE,0,2,1,&current_section);
    if( out==0 ) {
      break;
    }

    for( i=0; i<out/2; i++ ) {
      scnd_buf[i*4] = play_buf[i];
      scnd_buf[i*4+1] = play_buf[i];
      scnd_buf[i*4+2] = play_buf[i];
      scnd_buf[i*4+3] = play_buf[i];
    }

    write( dsp_fd, scnd_buf, out*4 );

    max = 0;
    for( i=0; i<out/2; i++ ) {
      if( play_buf[i]>max ) max=play_buf[i];
      else if( -play_buf[i]>max ) max=play_buf[i];
    }
    m = ((float)max)/32768.0f;
    if( m>1.0f ) m=1.0f;
    else if( m<0.0f ) m=0.0f;
    max_volume = m;

  }

  ov_clear(&vf);
  //fclose( oggfile );
  max_volume = 0.0f;

  finished_playing = 1;
  return NULL;
}

gint play_timer_callback( gpointer data )
{
  if( finished_playing && play_running ) {
    stop_play();
  }
  return TRUE;
}

void start_play()
{
  int r;

  play_running = 1;
  finished_playing = 0;
  if( record_running ) stop_record();
  if( monitor_running ) stop_monitor();

  vol_bar = lookup_widget( composer, "volume_bar" );
  max_volume = 0.0f;
  run_thread = 1;

  r = open_dsp( OPEN_PLAY );
  if( r==0 ) {
    timeout = gtk_timeout_add( 50, update_volume_bar_callback, NULL );
  
    pthread_create( &play_thread, NULL, play_loop, NULL ); 

    play_timer = gtk_timeout_add( 100, play_timer_callback, NULL );
    status_text( "Playing ..." );

  } else {
    status_text( "Error opening sound device /dev/dsp" );
  }

}

void stop_play()
{
  GtkWidget* togglebutton;

  run_thread = 0;
  pthread_join( play_thread, NULL );
  close_dsp();
  gtk_timeout_remove( timeout );
  gtk_timeout_remove( play_timer );

  togglebutton = lookup_widget( composer, "play_button" );
  if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(togglebutton) ) ) {
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(togglebutton), 0 );
  }
  play_running = 0;

  status_text( "" );
}

#define OGG_READ 1024

int save_ogg_file( char* filename, char* rec_buf, int rec_len )
{
  ogg_stream_state os; /* take physical pages, weld into a logical
			  stream of packets */
  ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
  ogg_packet       op; /* one raw packet of data for decode */
  
  vorbis_info      vi; /* struct that stores all the static vorbis bitstream
			  settings */
  vorbis_comment   vc; /* struct that stores all the user comments */

  vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
  vorbis_block     vb; /* local working space for packet->PCM decode */

  int eos=0;
  
  int buf_pos;
  //signed short* inbuf = (signed short*)rec_buf;
  signed short readbuffer[OGG_READ];

  FILE* fileout;

  fileout = fopen( filename, "w" );
  if( !fileout ) return 0;

  /********** Encode setup ************/

  /* choose an encoding mode */
  /* (quality mode .4: 44kHz stereo coupled, roughly 128kbps VBR) */
  vorbis_info_init(&vi);
  vorbis_encode_init_vbr(&vi,1,22050,.01); // max compression

  /* add a comment */
  vorbis_comment_init(&vc);
  vorbis_comment_add_tag(&vc,"be_a","Greporter");

  /* set up the analysis state and auxiliary encoding storage */
  vorbis_analysis_init(&vd,&vi);
  vorbis_block_init(&vd,&vb);
  
  /* set up our packet->stream encoder */
  /* pick a random serial number; that way we can more likely build
     chained streams just by concatenation */
  srand(time(NULL));
  ogg_stream_init(&os,rand());

  /* Vorbis streams begin with three headers; the initial header (with
     most of the codec setup parameters) which is mandated by the Ogg
     bitstream spec.  The second header holds any comment fields.  The
     third header holds the bitstream codebook.  We merely need to
     make the headers, then pass them to libvorbis one at a time;
     libvorbis handles the additional Ogg bitstream constraints */

  {
    ogg_packet header;
    ogg_packet header_comm;
    ogg_packet header_code;

    vorbis_analysis_headerout(&vd,&vc,&header,&header_comm,&header_code);
    ogg_stream_packetin(&os,&header); /* automatically placed in its own
					 page */
    ogg_stream_packetin(&os,&header_comm);
    ogg_stream_packetin(&os,&header_code);

    /* We don't have to write out here, but doing so makes streaming 
     * much easier, so we do, flushing ALL pages. This ensures the actual
     * audio data will start on a new page
     */
    while(!eos){
      int result=ogg_stream_flush(&os,&og);
      if(result==0)break;
      fwrite(og.header,1,og.header_len,fileout);
      fwrite(og.body,1,og.body_len,fileout);
    }

  }
  
  buf_pos = 0;
  while(!eos){
    long i;
    long bytes;
    //long bytes=fread(readbuffer,1,READ*4,stdin); /* stereo hardwired here */

    bytes = OGG_READ;
    if( bytes>rec_len-buf_pos ) bytes = rec_len-buf_pos;
    memcpy( readbuffer, &(rec_buf[buf_pos]), bytes );
    buf_pos += bytes;

    if(bytes==0){
      /* end of file.  this can be done implicitly in the mainline,
         but it's easier to see here in non-clever fashion.
         Tell the library we're at end of stream so that it can handle
         the last frame and mark end of stream in the output properly */
      vorbis_analysis_wrote(&vd,0);

    }else{
      /* data to encode */

      /* expose the buffer to submit data */
      float **buffer=vorbis_analysis_buffer(&vd,OGG_READ);
      
      /* uninterleave samples */
      /*for(i=0;i<bytes/4;i++){
        buffer[0][i]=((readbuffer[i*4+1]<<8)|
	              (0x00ff&(int)readbuffer[i*4]))/32768.f;
	buffer[1][i]=((readbuffer[i*4+3]<<8)|
		      (0x00ff&(int)readbuffer[i*4+2]))/32768.f;
      }*/
      for( i=0; i<bytes/2; i++ ) {
	buffer[0][i] = readbuffer[i]/32768.f;
      }

      /* tell the library how much we actually submitted */
      vorbis_analysis_wrote(&vd,i);
    }

    /* vorbis does some data preanalysis, then divvies up blocks for
       more involved (potentially parallel) processing.  Get a single
       block for encoding now */
    while(vorbis_analysis_blockout(&vd,&vb)==1){

      /* analysis, assume we want to use bitrate management */
      vorbis_analysis(&vb,NULL);
      vorbis_bitrate_addblock(&vb);

      while(vorbis_bitrate_flushpacket(&vd,&op)){
	
	/* weld the packet into the bitstream */
	ogg_stream_packetin(&os,&op);
	
	/* write out pages (if any) */
	while(!eos){
	  int result=ogg_stream_pageout(&os,&og);
	  if(result==0)break;
	  fwrite(og.header,1,og.header_len,fileout);
	  fwrite(og.body,1,og.body_len,fileout);
	  
	  /* this could be set above, but for illustrative purposes, I do
	     it here (to show that vorbis does know where the stream ends) */
	  
	  if(ogg_page_eos(&og))eos=1;
	}
      }
    }
  }

  /* clean up and exit.  vorbis_info_clear() must be called last */
  
  ogg_stream_clear(&os);
  vorbis_block_clear(&vb);
  vorbis_dsp_clear(&vd);
  vorbis_comment_clear(&vc);
  vorbis_info_clear(&vi);

  fclose( fileout );

  /* ogg_page and ogg_packet structs always point to storage in
     libvorbis.  They're never freed or manipulated directly */
  
  //fprintf(stderr,"Done.\n");  
  return 1;
}
