/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/
/* Protions from ogginfo
 * Copyright 2001, JAmes Atwill <ogg@linuxstuff.org>
 *
 * Portions from libvorbis examples, (c) Monty <monty@xiph.org>
 */

#define CVS_VERSION "$Id: reporter.c,v 1.5 2002/08/14 11:21:12 cvs Exp $"

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "support.h"
#include "reporter.h"
#include <mpn/matrix.h>
#include <mpn/matrixpublish.h>
#include <ogg/ogg.h>
#include <vorbis/codec.h>

char TEMP_OGG_FILE[MPN_STRING_LEN];

GtkWidget *composer;
int timer;
float progress_val;
int expiredelta;

struct any_item {
  int id;
  char name[MPN_STRING_LEN];
};

struct any_item *topic_list;
int topic_list_len;
int default_topic_index;

struct any_item *priority_list;
int priority_list_len;
int default_priority_index;

struct any_item *lang_list;
int lang_list_len;
int default_lang_index;



void init()
{
  mpn_init();
  sprintf( TEMP_OGG_FILE, "%s/tmp_audio.ogg", mpn_get_workdir() );
  topic_list = NULL;
  priority_list = NULL;
  lang_list = NULL;
}

void cleanup()
{
  if( topic_list ) free( topic_list );
  if( priority_list ) free( priority_list );
  if( lang_list ) free( lang_list );
  mpn_cleanup();
}

void apply_prefs()
{
  mpn_prefs_ptr prefsdoc;
  mpn_id_name_list_ptr idnamelist;
  mpn_id_name_ptr idname;
  GtkWidget* option_menu;
  GtkWidget* menu;
  GtkWidget* menuitem;
  GtkWidget* fileentry;
  GtkWidget* togglebutton;
  int i, r;

  // Read Prefs
  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "error reading prefs\n" );
    return;
  }

  topic_list_len = 0;
  priority_list_len = 0;
  lang_list_len = 0;
  default_topic_index = 0;
  default_priority_index = 0;
  default_lang_index = 0;
  if( topic_list ) free( topic_list );
  if( priority_list ) free( priority_list );
  if( lang_list ) free( lang_list );

  idnamelist = mpn_prefs_get_idnamelist( prefsdoc, "category" );
  if( idnamelist ) {
    topic_list_len = mpn_idnamelist_get_len( idnamelist );
    default_topic_index = mpn_idnamelist_get_default( idnamelist );
    topic_list = malloc( sizeof( struct any_item ) * topic_list_len );
    for( i=0; i<topic_list_len; i++ ) {
      idname = mpn_idnamelist_get_idname( idnamelist, i );
      topic_list[i].id = mpn_idname_get_id( idname );
      strcpy( topic_list[i].name, mpn_idname_get_name( idname ) );
    }
  }
  idnamelist = mpn_prefs_get_idnamelist( prefsdoc, "priority" );
  if( idnamelist ) {
    priority_list_len = mpn_idnamelist_get_len( idnamelist );
    default_priority_index = mpn_idnamelist_get_default( idnamelist );
    priority_list = malloc( sizeof( struct any_item ) * priority_list_len );
    for( i=0; i<priority_list_len; i++ ) {
      idname = mpn_idnamelist_get_idname( idnamelist, i );
      priority_list[i].id = mpn_idname_get_id( idname );
      strcpy( priority_list[i].name, mpn_idname_get_name( idname ) );
    }
  }
  idnamelist = mpn_prefs_get_idnamelist( prefsdoc, "language" );
  if( idnamelist ) {
    lang_list_len = mpn_idnamelist_get_len( idnamelist );
    default_lang_index = mpn_idnamelist_get_default( idnamelist );
    lang_list = malloc( sizeof( struct any_item ) * lang_list_len );
    for( i=0; i<lang_list_len; i++ ) {
      idname = mpn_idnamelist_get_idname( idnamelist, i );
      lang_list[i].id = mpn_idname_get_id( idname );
      strcpy( lang_list[i].name, mpn_idname_get_name( idname ) );
    }
  }

  fileentry = lookup_widget( composer, "location_text" );
  gtk_entry_set_text( (GtkEntry*)fileentry, mpn_prefs_get_defaultlocation( prefsdoc ) ); 

  fileentry = lookup_widget( composer, "subject_text" );
  gtk_entry_set_text( (GtkEntry*)fileentry, "" ); 

  mpn_prefs_close( prefsdoc );

  // Set Prefs to GUI
  option_menu = lookup_widget( composer, "topic_menu" );
  menu = gtk_menu_new ();
  for( i=0; i<topic_list_len; i++ ) {
    menuitem = gtk_menu_item_new_with_label ( topic_list[i].name );
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (menu), menuitem);
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 
			       default_topic_index );
			       
  option_menu = lookup_widget( composer, "priority_menu" );
  menu = gtk_menu_new ();
  for( i=0; i<priority_list_len; i++ ) {
    menuitem = gtk_menu_item_new_with_label ( priority_list[i].name );
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (menu), menuitem);
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 
			       default_priority_index );
			       
  option_menu = lookup_widget( composer, "language_menu" );
  menu = gtk_menu_new ();
  for( i=0; i<lang_list_len; i++ ) {
    menuitem = gtk_menu_item_new_with_label ( lang_list[i].name );
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (menu), menuitem);
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 
			       default_lang_index );
			       
  gtk_editable_delete_text ( GTK_EDITABLE(lookup_widget( 
			   composer, "comment_text" )), 0, -1 );

  fileentry = lookup_widget( composer, "fileentry" );
  gtk_entry_set_text( (GtkEntry*)fileentry, "" ); 

  expiredelta = 60*60*24;
  togglebutton = lookup_widget( composer, "oneday_radio" );
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(togglebutton), 1 );

  status_text( CVS_VERSION );
}

int get_option_menu_index( char* widget_name )
{
  GtkWidget *option_menu, *menu, *active_item;
  gint active_index;

  option_menu = lookup_widget (composer, widget_name );
  menu = GTK_OPTION_MENU (option_menu)->menu;
  active_item = gtk_menu_get_active (GTK_MENU (menu));
  active_index = g_list_index (GTK_MENU_SHELL (menu)->children,
active_item);

  return active_index;
}

/* Test the integrity of the stream header.  
   Return:
     1 if it is good
     0 if it is corrupted and os, vi, and vc were initialized
    -1 if it is corrupted and os, vi, and vc were not initialized 
       (don't clear them) */
int test_header (FILE *fp, ogg_sync_state *oy, ogg_stream_state *os,
		 vorbis_info *vi, vorbis_comment  *vc, long *serialno)
{
  ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
  ogg_packet       op; /* one raw packet of data for decode */
  char *buffer;
  int bytes;
  int i;
  
  /* grab some data at the head of the stream.  We want the first page
     (which is guaranteed to be small and only contain the Vorbis
     stream initial header) We need the first page to get the stream
     serialno. */
  
  /* submit a 4k block to libvorbis' Ogg layer */
  buffer=ogg_sync_buffer(oy,4096);
  bytes=fread(buffer,1,4096,fp);
  ogg_sync_wrote(oy,bytes);
  
  /* Get the first page. */
  if(ogg_sync_pageout(oy,&og)!=1){
    /* error case.  Must not be Vorbis data */
    return -1;
  }
  
  /* Get the serial number and set up the rest of decode. */
  /* serialno first; use it to set up a logical stream */
  *serialno = ogg_page_serialno(&og);
  ogg_stream_init(os, *serialno);
  
  /* extract the initial header from the first page and verify that the
     Ogg bitstream is in fact Vorbis data */
  
  /* I handle the initial header first instead of just having the code
     read all three Vorbis headers at once because reading the initial
     header is an easy way to identify a Vorbis bitstream and it's
     useful to see that functionality seperated out. */
  
  vorbis_info_init(vi);
  vorbis_comment_init(vc);
  if(ogg_stream_pagein(os,&og)<0){ 
    /* error; stream version mismatch perhaps */
    return 0;
  }
    
  if(ogg_stream_packetout(os,&op)!=1){ 
    /* no page? must not be vorbis */
    return 0;
  }
  
  if(vorbis_synthesis_headerin(vi,vc,&op)<0){ 
    /* error case; not a vorbis header */
    return 0;
  }
    
  /* At this point, we're sure we're Vorbis.  We've set up the logical
     (Ogg) bitstream decoder.  Get the comment and codebook headers and
     set up the Vorbis decoder */
  
  /* The next two packets in order are the comment and codebook headers.
     They're likely large and may span multiple pages.  Thus we reead
     and submit data until we get our two pacakets, watching that no
     pages are missing.  If a page is missing, error out; losing a
     header page is the only place where missing data is fatal. */
  
  i=0;
  while(i<2){
    while(i<2){
      int result=ogg_sync_pageout(oy,&og);
      if(result==0)break; /* Need more data */
      /* Don't complain about missing or corrupt data yet.  We'll
	 catch it at the packet output phase */
      if(result==1){
	ogg_stream_pagein(os,&og); /* we can ignore any errors here
				       as they'll also become apparent
				       at packetout */
	while(i<2){
	  result=ogg_stream_packetout(os,&op);
	  if(result==0)break;
	  if(result<0){
	    /* Uh oh; data at some point was corrupted or missing!
	       We can't tolerate that in a header. */
	    return 0;
	  }
	  vorbis_synthesis_headerin(vi,vc,&op);
	  i++;
	}
      }
    }

    /* no harm in not checking before adding more */
    buffer=ogg_sync_buffer(oy,4096);
    bytes=fread(buffer,1,4096,fp);
    if(bytes==0 && i<2){
      return 0;
    }
    ogg_sync_wrote(oy,bytes);
  }
  
  /* If we made it this far, the header must be good. */
  return 1;
}


/* Test the integrity of the vorbis stream after the header.
   Return:
     >0 if the stream is correct and complete
     0 if the stream is correct but truncated
    -1 if the stream is corrupted somewhere in the middle */
int test_stream (FILE *fp, ogg_sync_state *oy, ogg_stream_state *os, 
		 ogg_int64_t *final_granulepos, ogg_int64_t *bits)
{
  int eos = 0;
  ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
  ogg_packet       op; /* one raw packet of data for decode */
  int bytes;
  char *buffer;

  /* Just a straight decode loop until end of stream */
    while(!eos){
      while(!eos){
	int result=ogg_sync_pageout(oy,&og);
	if(result==0)break; /* need more data */
	if(result<0){ /* missing or corrupt data at this page position */
	  return -1;
	}else{
	  if (ogg_stream_pagein(os,&og) < 0) {
	    return -1;
	  }

	  while(1){
	    result=ogg_stream_packetout(os,&op);

	    if(result==0)break; /* need more data */
	    if(result<0){ /* missing or corrupt data at this page position */
	      return -1;
	    }else{
	      /* Normally we would do decode here, but we're just checking
		 packet integrity */
	    }
	  }
	  if(ogg_page_eos(&og))eos=1;
	}
      }
      if(!eos){
	buffer = ogg_sync_buffer(oy,4096);
	bytes = fread(buffer,1,4096,fp);
	*bits += 8 * bytes;
	ogg_sync_wrote(oy,bytes);
	if(bytes == 0) 
	  eos = 1;
      }
    }

    *final_granulepos = ogg_page_granulepos(&og);

    /* Make sure that the last page is marked as the end-of-stream */
    return ogg_page_eos(&og);
}

int get_ogg_specific( const char* filename, int* playtime_r, int* rate_r, 
		      int* channels_r )
{
  int header_state = -1; /* Assume the worst */
  int stream_state = -1;

  ogg_sync_state   oy; /* sync and verify incoming physical bitstream */
  ogg_stream_state os; /* take physical pages, weld into a logical
			  stream of packets */
  vorbis_info      vi; /* struct that stores all the static vorbis bitstream
			  settings */
  vorbis_comment   vc; /* struct that stores all the bitstream user comments */
  
  FILE *fp;
  int eof = 0;
  long serialno;
  ogg_int64_t bits, final_granulepos;
  double playtime, total_playtime = 0.0;

  /********** Decode setup ************/

  fp = fopen(filename,"rb");
  if (!fp) {
    fprintf(stderr,"Unable to open \"%s\"\n",
	    filename );
    return 0;
  }

  ogg_sync_init(&oy); /* Now we can read pages */
  
  while (!eof) {
    bits = 0;

    if ( (header_state = test_header(fp, &oy, &os, &vi, &vc, 
				     &serialno)) == 1 ) {
      stream_state = test_stream(fp, &oy, &os, &final_granulepos, 
				 &bits);
      *rate_r = vi.rate;
      *channels_r = vi.channels;
    }

    if (stream_state >= 0) {
      playtime = (double) final_granulepos / vi.rate;
      total_playtime += playtime;
    }
    
    /* clean up this logical bitstream; before exit we see if we're
       followed by another [chained] */
    eof = feof(fp);
  }
  
  if (header_state >= 0) {

    /* We got far enough to initialize these structures */
    ogg_stream_clear(&os);
      
    /* ogg_page and ogg_packet structs always point to storage in
       libvorbis.  They're never freed or manipulated directly */
    
    vorbis_comment_clear(&vc);
    vorbis_info_clear(&vi);  /* must be called last */
  }
  
  /* OK, clean up the framer */
  ogg_sync_clear(&oy);
  
  fclose(fp);

  *playtime_r = (int)total_playtime;
  return MPN_OK;
}

void broadcast()
{
  mpn_news_ptr newsdoc;
  mpn_newsitem_ptr newsitem;
  mpn_app_ptr app;
  mpn_app_block_ptr appblock;
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr identity;
  mpn_docinfo_ptr docinfo;
  long expire;
  char filename[MPN_STRING_LEN];
  FILE* mfile;
  long filesize;
  char* mbuf;
  GtkWidget *fileentry;
  gchar* comment;
  int r, rate, channels;
  int playtime;
  char typecont[MPN_STRING_LEN];
  gchar* subject;
  gchar* location;
  char log[MPN_STRING_LEN];
  mpn_news_ptr lognewsdoc;

  status_text( "Publishing message ..." );
  status_activity_on();

  // Get Identity
  r = mpn_prefs_open_private( MPN_OPEN_READONLY, &prefsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "Error reading prefsfile\n" );
    status_text( "Error reading prefsfile." );
    status_activity_off();
    return;
  }
  identity = mpn_prefs_get_identity( prefsdoc, 
		     mpn_prefs_get_identity_default( prefsdoc ) );

  // Read Messagefile
  fileentry = lookup_widget( composer, "fileentry" );
  strcpy( filename, (char*)(gtk_entry_get_text( (GtkEntry*)fileentry ) ) );

  mfile = fopen( filename, "r" ); 
  if( !mfile ) {
    status_text( "Error: Soundfile does not exist." );
    status_activity_off();
    fprintf( stderr, "ogg sound file is not existant\n" );
    fprintf( stderr, "not sending anything\n" );
    return;
  }
  fseek( mfile, 0, SEEK_END );
  filesize = ftell( mfile );
  mbuf = malloc( filesize );
  rewind( mfile );
  fread( mbuf, 1, filesize, mfile );
  fclose( mfile );

  expire = time( NULL )+expiredelta;

  // audio/ogg specific
  get_ogg_specific( filename, &playtime, &rate, &channels );
  sprintf( typecont, "\
<duration>%d</duration>\n\
<rate>%d</rate>\n\
<channels>%d</channels>", 
	   playtime, rate, channels );

  // subject and location
  subject = gtk_editable_get_chars ( GTK_EDITABLE(lookup_widget( 
				composer, "subject_text" )), 0, -1 );
  location = gtk_editable_get_chars ( GTK_EDITABLE(lookup_widget( 
				composer, "location_text" )), 0, -1 );

  // Make Newsitem
  r = mpn_news_compose_default_file( identity, &newsdoc, "greporter", ".ogg",
				     mbuf, filesize, "audio/ogg", typecont, 
				     expire, subject, location );
  g_free( location );
  g_free( subject );
  free( mbuf );

  if( r!=MPN_OK ) {
    status_activity_off();
    status_text( "Error composing message." );
    fprintf( stderr, "error composing message\n" );
    fprintf( stderr, "not sending anything\n" );
    return;
  }

  // Set Reporter Data
  newsitem = mpn_news_get_item( newsdoc, 0 );
  app = mpn_newsitem_get_app( newsitem );
  docinfo = mpn_news_get_docinfo( newsdoc );

  appblock = mpn_app_add_block( app, "reporter" );
  mpn_app_block_set_int( docinfo, appblock, "category_id", 
			 topic_list[get_option_menu_index( "topic_menu" )].id);
  mpn_app_block_set_int( docinfo, appblock, "language_id", 
		       lang_list[get_option_menu_index( "language_menu" )].id);
  mpn_app_block_set_int( docinfo, appblock, "priority_id", 
		   priority_list[get_option_menu_index( "priority_menu" )].id);
  
  comment = gtk_editable_get_chars ( GTK_EDITABLE(lookup_widget( 
				composer, "comment_text" )), 0, -1 );
  mpn_app_block_set_string( docinfo, appblock, "comment", comment );
  g_free( comment );

  // Send Message
  r = mpn_publish_newsitem_file( identity, newsdoc, newsitem, filename );
  if( r==MPN_OK ) {
    status_activity_off();
    status_text( "Message published." );
  } else {
    status_activity_off();
    status_text( "Error publishing message." );
    fprintf( stderr, "error publishing message\n" );
  }

  // Write to log-file
  sprintf( log, "%s/%s", mpn_get_workdir(), "published.xml" );
  r = mpn_news_open_foreign( log, MPN_OPEN_READWRITE, &lognewsdoc, "matrix-published-news" );
  if( r==MPN_OK ) {
    mpn_news_add_newsdoc( lognewsdoc, newsdoc );    
    mpn_news_close( lognewsdoc );
  }

  mpn_news_close( newsdoc );
  return;
}
 
void clear()
{
  apply_prefs();
}

void status_text( const char* text )
{
  GtkWidget* statusbar;

  statusbar = lookup_widget( composer, "status_text" );
  gtk_label_set_text( statusbar, (const gchar*)text );  
}

gint status_activity_do( gpointer data )
{
  /*
  GtkWidget* statusbar;

  statusbar = lookup_widget( composer, "composer_statusbar" );
  gtk_progress_set_value( (GtkProgress*)gnome_appbar_get_progress( (GnomeAppBar*)statusbar ), progress_val+=0.01f );
  */
  return TRUE;
}

void status_activity_on()
{
  /*
  GtkWidget* statusbar;

  progress_val = 0.0f;
  statusbar = lookup_widget( composer, "composer_statusbar" );

  gtk_progress_set_activity_mode(
		 gnome_appbar_get_progress( (GnomeAppBar*)statusbar ),
		 1 );

  gtk_progress_bar_set_activity_step( gnome_appbar_get_progress( (GnomeAppBar*)statusbar ),
				      20 );


  timer = gtk_timeout_add( 100, status_activity_do, NULL );
  */
}

void status_activity_off()
{
  /*
  GtkWidget* statusbar;

  gtk_timeout_remove( timer );

  statusbar = lookup_widget( composer, "composer_statusbar" );
  gtk_progress_set_activity_mode(
		 gnome_appbar_get_progress( (GnomeAppBar*)statusbar ),
		 0 );

  gtk_progress_set_percentage(
		 gnome_appbar_get_progress( (GnomeAppBar*)statusbar ),
		 0.0f );
  */
}

