/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>, Nils Behrens <psi@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

/* This file is part of libmatrix
** It grants access to the preferences and news data
*/

#ifndef __MATRIX_PUBLIC_NET__
#define __MATRIX_PUBLIC_NET__



#include <libxml/parser.h>
#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>





/* global things */

#define MPN_STRING_LEN   200

#define MPN_OK             0
#define MPN_PERM_ERR       1
#define MPN_TEMP_ERR       2

#define MPN_OPEN_READONLY  1
#define MPN_OPEN_READWRITE 2

#define MPN_NEWS_PUBLISH   0
#define MPN_NEWS_RECOMMEND 1
#define MPN_NEWS_PRIVATE   2
#define MPN_NEWS_PUBLIC    3
#define MPN_NEWS_TEMP      4
#define MPN_NEWS_UNKNOWN   5


int mpn_init();
int mpn_cleanup();

const char* mpn_get_workdir();
const char* mpn_get_newsfile();
const char* mpn_get_prefsfile();
const char* mpn_get_contentdir();

/* NEWS NEWS NEWS NEWS NEWS */

/* News Data Structures */

/* KEY Section */
struct _mpn_key;
typedef struct _mpn_key mpn_key;
typedef mpn_key* mpn_key_ptr;

/* SIG Section */
struct _mpn_file;
typedef struct _mpn_file mpn_file;
typedef mpn_file* mpn_file_ptr;

struct _mpn_person;
typedef struct _mpn_person mpn_person;
typedef mpn_person* mpn_person_ptr;

struct _mpn_date;
typedef struct _mpn_date mpn_date;
typedef mpn_date* mpn_date_ptr;

struct _mpn_sig;
typedef struct _mpn_sig mpn_sig;
typedef mpn_sig* mpn_sig_ptr;

/* .NET Section */
struct _mpn_mirror;
typedef struct _mpn_mirror mpn_mirror;
typedef mpn_mirror* mpn_mirror_ptr;

struct _mpn_mirror_list;
typedef struct _mpn_mirror_list mpn_mirror_list;
typedef mpn_mirror_list* mpn_mirror_list_ptr;

struct _mpn_net;
typedef struct _mpn_net mpn_net;
typedef mpn_net* mpn_net_ptr;

/* APP Section */
struct _mpn_app_block;
typedef struct _mpn_app_block mpn_app_block;
typedef mpn_app_block* mpn_app_block_ptr;

struct _mpn_app_list;
typedef struct _mpn_app_list mpn_app_list;
typedef mpn_app_list* mpn_app_list_ptr;

struct _mpn_app;
typedef struct _mpn_app mpn_app;
typedef mpn_app* mpn_app_ptr;

/* Main News */
struct _mpn_newsitem;
typedef struct _mpn_newsitem mpn_newsitem;
typedef mpn_newsitem* mpn_newsitem_ptr;

struct _mpn_newslist;
typedef struct _mpn_newslist mpn_newslist;
typedef mpn_newslist* mpn_newslist_ptr;

struct _mpn_docinfo;
typedef struct _mpn_docinfo mpn_docinfo;
typedef mpn_docinfo* mpn_docinfo_ptr;

struct _mpn_news;
typedef struct _mpn_news mpn_news;
typedef mpn_news* mpn_news_ptr;

/* Filter Doc */
//struct _mpn_xslfilterdoc;
//typedef struct _mpn_xslfilterdoc mpn_xslfilterdoc;
//typedef mpn_xslfilterdoc* mpn_xslfilterdoc_ptr;

/* PREFS PREFS PREFS PREFS PREFS */

/* Prefs Data Structures, DO NOT USE DIRECTLY */

/* Generic ID - Name Lists for a whole bunch of apps */
struct _mpn_id_name;
typedef struct _mpn_id_name mpn_id_name;
typedef mpn_id_name* mpn_id_name_ptr;

struct _mpn_id_name_list;
typedef struct _mpn_id_name_list mpn_id_name_list;
typedef mpn_id_name_list* mpn_id_name_list_ptr;

struct _mpn_all_id_name_lists;
typedef struct _mpn_all_id_name_lists mpn_all_id_name_lists;
typedef mpn_all_id_name_lists* mpn_all_id_name_lists_ptr;

/* Find out about your friends */
struct _mpn_friend;
typedef struct _mpn_friend mpn_friend;
typedef mpn_friend* mpn_friend_ptr;

struct _mpn_friend_list;
typedef struct _mpn_friend_list mpn_friend_list;
typedef mpn_friend_list* mpn_friend_list_ptr;

/* Newsfilter */
struct _mpn_filter;
typedef struct _mpn_filter mpn_filter;
typedef mpn_filter* mpn_filter_ptr;

/* Newsblocks */
struct _mpn_newsblock;
typedef struct _mpn_newsblock mpn_newsblock;
typedef mpn_newsblock* mpn_newsblock_ptr;

struct _mpn_newsblock_list;
typedef struct _mpn_newsblock_list mpn_newsblock_list;
typedef mpn_newsblock_list* mpn_newsblock_list_ptr;

/* Identities */
struct _mpn_local_account;
typedef struct _mpn_local_account mpn_local_account;
typedef mpn_local_account* mpn_local_account_ptr;

struct _mpn_remote_account;
typedef struct _mpn_remote_account mpn_remote_account;
typedef mpn_remote_account* mpn_remote_account_ptr;

struct _mpn_identity;
typedef struct _mpn_identity mpn_identity;
typedef mpn_identity* mpn_identity_ptr;

struct _mpn_identity_list;
typedef struct _mpn_identity_list mpn_identity_list;
typedef mpn_identity_list* mpn_identity_list_ptr;

/* All Preferences for the glorious Matrix Public Net*/
struct _mpn_prefs;
typedef struct _mpn_prefs mpn_prefs;
typedef mpn_prefs* mpn_prefs_ptr;

/* NEWS NEWS NEWS NEWS NEWS */

/* Dealing with news files */

void mpn_news_open_empty( mpn_news_ptr* newsdoc_ret );
int mpn_news_open_private( int mode, mpn_news_ptr* newsdoc_ret );
int mpn_news_open_foreign( const char* filename, int mode, 
			   mpn_news_ptr* newsdoc_ret, const char* root ); 
int mpn_news_open_mem( const char* buf, mpn_news_ptr* newsdoc_ret );

void mpn_news_close( mpn_news_ptr newsdoc );

int mpn_fetch_newslist( const char* url, const char* cookie, 
			const char* filename );

/* Dealing with news */

mpn_docinfo_ptr mpn_news_get_docinfo( mpn_news_ptr newsdoc );
int mpn_news_get_type( mpn_news_ptr newsdoc );
const char* mpn_news_get_rootnode( mpn_news_ptr newsdoc );
const char* mpn_news_get_username( mpn_news_ptr newsdoc );
int mpn_news_get_mailreply( mpn_news_ptr newsdoc );
const char* mpn_news_get_cookie( mpn_news_ptr newsdoc );
const char* mpn_news_get_filename( mpn_news_ptr newsdoc );
int mpn_news_get_mode( mpn_news_ptr newsdoc );
int mpn_news_get_len( mpn_news_ptr newsdoc );
mpn_newsitem_ptr mpn_news_get_item( mpn_news_ptr newsdoc, int i );
void mpn_news_add_item( mpn_news_ptr newsdoc, mpn_newsitem_ptr item );
void mpn_news_del_item( mpn_news_ptr newsdoc, mpn_newsitem_ptr item );
int mpn_news_filter( mpn_news_ptr newsdoc, mpn_filter_ptr filter,
		     mpn_news_ptr* newsdoc_ret, int verbose );
void mpn_news_add_newsdoc( mpn_news_ptr newsdoc, mpn_news_ptr adddoc );
int mpn_news_compose_default( mpn_identity_ptr identity,
			      mpn_news_ptr* newsdoc_ret,
			      const char* idpart, const char* idpart2,
			      long expire,
			      const char* subject, const char* location );
int mpn_news_compose_default_file( mpn_identity_ptr identity,
				   mpn_news_ptr* newsdoc_ret,
				   const char* idpart, const char* idpart2,
				   const char* filebuf, 
				   long filesize, const char* type, 
				   const char* typecont, long expire,
				   const char* subject, const char* location );
int mpn_news_compose_default_url( mpn_identity_ptr identity,
				  mpn_news_ptr* newsdoc_ret,
				  const char* idpart, const char* idpart2,
				  const char* url, 
				  const char* type, const char* typecont, 
				  long expire,
				  const char* subject, const char* location );

/* Dealing with newsitems */

mpn_key_ptr mpn_newsitem_get_key( mpn_newsitem_ptr item );
mpn_sig_ptr mpn_newsitem_get_sig( mpn_newsitem_ptr item );
mpn_net_ptr mpn_newsitem_get_net( mpn_newsitem_ptr item );
mpn_app_ptr mpn_newsitem_get_app( mpn_newsitem_ptr item );
int mpn_newsitem_equal( mpn_newsitem_ptr i1, mpn_newsitem_ptr i2 );
int mpn_newsitem_filter( mpn_newsitem_ptr item, mpn_filter_ptr filter,
			 mpn_news_ptr* newsdoc_ret, int verbose );
void mpn_newsitem_dump( mpn_news_ptr newsdoc, mpn_newsitem_ptr item,
			char* buf );
int mpn_newsitem_fetch_file( mpn_news_ptr newsdoc, mpn_newsitem_ptr item, 
			     char* filename_ret );

/* Dealing with keys */

const char* mpn_key_get_signature( mpn_key_ptr key );

/* Dealing with sigs */

const char* mpn_sig_get_id( mpn_sig_ptr sig );
mpn_file_ptr mpn_sig_get_file( mpn_sig_ptr sig );
mpn_person_ptr mpn_sig_get_author( mpn_sig_ptr sig );
mpn_date_ptr mpn_sig_get_expire( mpn_sig_ptr sig );

/* Dealing with files */

const char* mpn_file_get_url( mpn_file_ptr file );
const char* mpn_file_get_hash( mpn_file_ptr file );
long mpn_file_get_size( mpn_file_ptr file );
const char* mpn_file_get_type( mpn_file_ptr file );
mpn_app_block_ptr mpn_file_get_typeblock( mpn_file_ptr file );

/* Dealing with persons */

const char* mpn_person_get_name( mpn_person_ptr person );
const char* mpn_person_get_home( mpn_person_ptr person );

/* Dealing with dates */

long mpn_date_get_sec( mpn_date_ptr date );
const char* mpn_date_get_txt( mpn_date_ptr date );

/* Dealing withs nets */

double mpn_net_get_weight( mpn_net_ptr net );
int mpn_net_get_ttl( mpn_net_ptr net );
int mpn_net_get_hop( mpn_net_ptr net );
int mpn_net_get_corrupt( mpn_net_ptr net );
int mpn_net_get_keep( mpn_net_ptr net );
int mpn_net_get_dontrecommend( mpn_net_ptr net );
const char* mpn_net_get_localfile( mpn_net_ptr net );
int mpn_net_get_mirror_count( mpn_net_ptr net );
const char* mpn_net_get_mirror( mpn_net_ptr net, int i );

void mpn_net_set_weight( mpn_news_ptr newsdoc, mpn_net_ptr net,double weight );
void mpn_net_set_ttl( mpn_news_ptr newsdoc, mpn_net_ptr net, int ttl );
void mpn_net_set_hop( mpn_news_ptr newsdoc, mpn_net_ptr net, int hop );
void mpn_net_set_corrupt( mpn_news_ptr newsdoc, mpn_net_ptr net, int corrupt );
void mpn_net_set_localfile( mpn_news_ptr newsdoc, mpn_net_ptr net, 
			    const char* localfile );
void mpn_net_add_mirror( mpn_news_ptr newsdoc, mpn_net_ptr net, 
			 const char* mirror );

/* Dealing with apps */

mpn_app_block_ptr mpn_app_get_block( mpn_app_ptr app, const char* name );
mpn_app_block_ptr mpn_app_add_block( mpn_app_ptr app, const char* name );

/* Dealing with appblocks */

int mpn_app_block_get_int( mpn_docinfo_ptr docinfo, mpn_app_block_ptr ablock, 
			   const char* name );
long mpn_app_block_get_long( mpn_docinfo_ptr docinfo, 
			     mpn_app_block_ptr ablock, const char* name );
void mpn_app_block_get_string( mpn_docinfo_ptr docinfo, 
			       mpn_app_block_ptr ablock, 
			       const char* name, char* val );
int mpn_app_block_get_filter( mpn_docinfo_ptr docinfo, 
			      mpn_app_block_ptr ablock, 
			      const char* name, mpn_filter_ptr* filter );

void mpn_app_block_set_int( mpn_docinfo_ptr docinfo, mpn_app_block_ptr ablock,
			    const char* name, int i );
void mpn_app_block_set_long( mpn_docinfo_ptr docinfo, mpn_app_block_ptr ablock,
			     const char* name, long l );
void mpn_app_block_set_string( mpn_docinfo_ptr docinfo, 
			       mpn_app_block_ptr ablock,
			       const char* name, const char* s );

int mpn_app_block_get_subblock_count( mpn_docinfo_ptr docinfo, 
				      mpn_app_block_ptr ablock,
				      const char* name );
int mpn_app_block_get_subblock( mpn_docinfo_ptr docinfo, 
				mpn_app_block_ptr ablock,
				const char* name, int i,
				mpn_app_block_ptr* subblock);
void mpn_app_block_free( mpn_app_block_ptr subblock );

/* PREFS PREFS PREFS PREFS PREFS */

/* Dealing with prefs files */

int mpn_prefs_open_private( int mode, mpn_prefs_ptr* prefsdoc );
void mpn_prefs_close( mpn_prefs_ptr prefsdoc );

/* Dealing with prefs */

mpn_docinfo_ptr mpn_prefs_get_docinfo( mpn_prefs_ptr prefsdoc );
int mpn_prefs_get_identity_count( mpn_prefs_ptr prefsdoc );
int mpn_prefs_get_identity_default( mpn_prefs_ptr prefsdoc );
mpn_identity_ptr mpn_prefs_get_identity( mpn_prefs_ptr prefsdoc, int i );
int mpn_prefs_del_identity( mpn_prefs_ptr prefsdoc, int i );
int mpn_prefs_add_remote_identity( mpn_prefs_ptr prefsdoc , const char* name, 
				   const char* home, const char* userid,
				   const char* email, const char* server_email,
				   const char* public_space,
				   const char* mailserver );

int mpn_prefs_get_friend_count( mpn_prefs_ptr prefsdoc );
mpn_friend_ptr mpn_prefs_get_friend( mpn_prefs_ptr prefsdoc, int i );
int mpn_prefs_add_friend( mpn_prefs_ptr prefsdoc, const char* name, 
			  const char* home, int poll, int bot,
			  const char* cookie );
int mpn_prefs_del_friend( mpn_prefs_ptr prefsdoc, int i );

mpn_id_name_list_ptr mpn_prefs_get_idnamelist( mpn_prefs_ptr prefsdoc, 
					       const char* name );
mpn_id_name_list_ptr mpn_prefs_add_idnamelist( mpn_prefs_ptr prefsdoc,
					       const char* name );

mpn_filter_ptr mpn_prefs_get_globalfilter( mpn_prefs_ptr prefsdoc );
mpn_filter_ptr mpn_prefs_get_deffriendfilter( mpn_prefs_ptr prefsdoc );
mpn_filter_ptr mpn_prefs_get_defbotfilter( mpn_prefs_ptr prefsdoc );

mpn_app_ptr mpn_prefs_get_app( mpn_prefs_ptr prefsdoc );
const char* mpn_prefs_get_defaultlocation( mpn_prefs_ptr prefsdoc );

/* Dealing with identities */

mpn_person_ptr mpn_identity_get_person( mpn_identity_ptr id );
mpn_local_account_ptr mpn_identity_get_local_account( mpn_identity_ptr id );
mpn_remote_account_ptr mpn_identity_get_remote_account( mpn_identity_ptr id );
const char* mpn_identity_get_public_space( mpn_identity_ptr id );
int mpn_identity_get_reserve( mpn_identity_ptr id );

/* Dealing with local accounts */

const char* mpn_local_account_get_news( mpn_local_account_ptr la );
const char* mpn_local_account_get_space( mpn_local_account_ptr la );

/* Dealing with remote accounts */

const char* mpn_remote_account_get_userid( mpn_remote_account_ptr ra );
const char* mpn_remote_account_get_email( mpn_remote_account_ptr ra );
const char* mpn_remote_account_get_mailserver( mpn_remote_account_ptr ra );
const char* mpn_remote_account_get_serveremail( mpn_remote_account_ptr ra );
int mpn_remote_account_get_mailreply( mpn_remote_account_ptr ra );

/* Dealing with friends */

mpn_person_ptr mpn_friend_get_person( mpn_friend_ptr friend );
int mpn_friend_get_poll( mpn_friend_ptr friend );
const char* mpn_friend_get_cookie( mpn_friend_ptr friend );
mpn_filter_ptr mpn_friend_get_filter( mpn_friend_ptr friend );
int mpn_friend_is_bot( mpn_friend_ptr friend );

void mpn_friend_set_cookie( mpn_prefs_ptr prefsdoc, mpn_friend_ptr friend, 
			    const char* cookie );

/* Dealing with ID-Name Lists */

int mpn_idnamelist_get_len( mpn_id_name_list_ptr idnamelist );
int mpn_idnamelist_get_default( mpn_id_name_list_ptr idnamelist );
void mpn_idnamelist_set_default( mpn_prefs_ptr prefsdoc,
				 mpn_id_name_list_ptr idnamelist, int i );
mpn_id_name_ptr mpn_idnamelist_get_idname( mpn_id_name_list_ptr idnamelist,
					   int i );
void mpn_idnamelist_add_entry( mpn_id_name_list_ptr idnamelist, int id,
			       const char* name );
void mpn_idnamelist_del_entry( mpn_id_name_list_ptr idnamelist, int i );

/* Dealing with ID-Names */

int mpn_idname_get_id( mpn_id_name_ptr idname );
const char* mpn_idname_get_name( mpn_id_name_ptr idname );


void mpn_filter_free( mpn_filter_ptr filter );


/////////////////////////////////
// MFL

#if (defined WIN32 || defined (_WIN32))
#   undef __WINDOWS__
#   define __WINDOWS__
#   undef __MSDOS__
#   define __MSDOS__
#   undef UNIX                          /*  Some Win32 compilers like RSXNT  */
#   undef unix                          /*    define UNIX as well as Win32   */
#   undef __unix__                      /*    which causes problems...       */
#endif

#if (defined WINDOWS || defined (_WINDOWS) || defined (__WINDOWS__))
#   undef __WINDOWS__
#   define __WINDOWS__
#   undef __MSDOS__
#   define __MSDOS__
#endif

/*  Try to define a __UTYPE_xxx symbol...                                    */
/*  unix                SunOS at least                                       */
/*  __unix__            gcc                                                  */
/*  _POSIX_SOURCE is various UNIX systems, maybe also VAX/VMS                */
#if (defined (unix) || defined (__unix__) || defined (_POSIX_SOURCE))
#   if (!defined (__VMS__))
#       undef __UNIX__
#       define __UNIX__
#       if (defined (__alpha))          /*  Digital UNIX is 64-bit           */
#           undef  __IS_32BIT__
#           define __IS_64BIT__
#           define __UTYPE_DECALPHA
#       endif
#   endif
#endif


void mfl_sleep(int msec);
int mfl_mkdir(char* dirname);

#endif

