/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>, Nils Behrens <psi@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef __MATRIX_PUBLIC_NET_INTERN__
#define __MATRIX_PUBLIC_NET_INTERN__




/* NEWS NEWS NEWS NEWS NEWS */

/* News Data Structures */

/* KEY Section */
struct _mpn_key {
  char key[MPN_STRING_LEN];
};

/* SIG Section */
struct _mpn_file {
  char url[MPN_STRING_LEN];
  char hash[MPN_STRING_LEN];
  long size;
  char type[MPN_STRING_LEN];
  mpn_app_block_ptr typeblock;
};

struct _mpn_person {
  char name[MPN_STRING_LEN];
  char home[MPN_STRING_LEN];
};

struct _mpn_date {
  long sec;
  char txt[MPN_STRING_LEN];
};

struct _mpn_sig {
  char id[MPN_STRING_LEN];
  mpn_file_ptr file;
  mpn_person_ptr author;
  mpn_date_ptr expire;
};

/* .NET Section */
struct _mpn_mirror {
  char url[MPN_STRING_LEN];
};

struct _mpn_mirror_list {
  mpn_mirror_ptr* list;
  int len;
};

struct _mpn_net {
  xmlNodePtr cur;
  double weight;
  int ttl;
  int hop;
  mpn_mirror_list_ptr mirrors;
  int corrupt;
  char localfile[MPN_STRING_LEN];
  int keep;
  int dontrecommend;
};

/* APP Section */
struct _mpn_app_block {
  char name[MPN_STRING_LEN];
  xmlNodePtr cur;
};

struct _mpn_app_list {
  int len;
  mpn_app_block_ptr* list;
};

struct _mpn_app {
  xmlNodePtr cur;
  mpn_app_list_ptr apps;
};

/* Main News */
struct _mpn_newsitem {
  mpn_key_ptr key;
  mpn_sig_ptr sig;
  mpn_net_ptr net;
  mpn_app_ptr app;
  xmlNodePtr cur;
};

struct _mpn_newslist {
  int len;
  mpn_newsitem_ptr* list;
};

struct _mpn_docinfo {
  xmlDocPtr doc;
};

struct _mpn_news {
  int mode;
  int news_type;
  char cookie[MPN_STRING_LEN];
  char username[MPN_STRING_LEN];
  int mailreply;
  mpn_newslist_ptr news;
  mpn_docinfo doc;
  xmlNodePtr cur;
  char filename[MPN_STRING_LEN];
  char root[MPN_STRING_LEN];
};

/* Filter Doc */
struct _mpn_xslfilterdoc {
  xmlDocPtr doc;
  xsltStylesheetPtr xsl;
};
typedef struct _mpn_xslfilterdoc mpn_xslfilterdoc;
typedef mpn_xslfilterdoc* mpn_xslfilterdoc_ptr;

/* PREFS PREFS PREFS PREFS PREFS */

/* Prefs Data Structures, DO NOT USE DIRECTLY */

/* Generic ID - Name Lists for a whole bunch of apps */
struct _mpn_id_name {
  int id;
  char name[MPN_STRING_LEN];
  xmlNodePtr cur;
};

struct _mpn_id_name_list {
  int len;
  int def;
  char name[MPN_STRING_LEN];
  mpn_id_name_ptr* list;
  xmlNodePtr cur;
};

struct _mpn_all_id_name_lists {
  int len;
  mpn_id_name_list_ptr* list;
};

/* Find out about your friends */
struct _mpn_friend {
  mpn_person_ptr person;
  int poll;
  char cookie[MPN_STRING_LEN];
  mpn_filter_ptr filter;
  xmlNodePtr cur;
  int bot;
};

struct _mpn_friend_list {
  int len;
  mpn_friend_ptr* list;
};

/* Newsfilter */
struct _mpn_filter {
  char xslprocess[MPN_STRING_LEN];
};

/* Identities */
struct _mpn_local_account {
  char news_file[MPN_STRING_LEN];
  char space_dir[MPN_STRING_LEN];
};

struct _mpn_remote_account {
  char userid[MPN_STRING_LEN];
  char email[MPN_STRING_LEN];
  char mailserver[MPN_STRING_LEN];
  char server_email[MPN_STRING_LEN];
  int mailreply;
};  

struct _mpn_identity {
  mpn_person_ptr person;
  mpn_local_account_ptr local;
  mpn_remote_account_ptr remote;
  char public_space[MPN_STRING_LEN];
  int reserve;
  xmlNodePtr cur;
};

struct _mpn_identity_list {
  int len;
  int def;
  mpn_identity_ptr* list;
};

/* All Preferences for the glorious Matrix Public Net*/

struct _mpn_prefs {
  int mode;
  mpn_docinfo doc;
  xmlNodePtr cur;
  char filename[MPN_STRING_LEN];

  mpn_identity_list_ptr identities;
  mpn_friend_list_ptr friends;
  mpn_all_id_name_lists_ptr id_name_lists;
  mpn_filter_ptr filter;
  mpn_filter_ptr def_friendfilter;
  mpn_filter_ptr def_botfilter;
  mpn_app_ptr app;
  char deflocation[MPN_STRING_LEN];
};

/* Function Prototypes */

int mpn_xslfilterdoc_open( const char* filename,
			   mpn_xslfilterdoc_ptr* filter_ret );
void mpn_xslfilterdoc_close( mpn_xslfilterdoc_ptr filterdoc );

int mpn_newsitem_xsl_process( mpn_newsitem_ptr item, 
			      mpn_xslfilterdoc_ptr xsldoc,
			      mpn_news_ptr* newsdoc_ret, int verbose );

const char* mpn_filter_get_xslprocess( mpn_filter_ptr f );
void mpn_filter_free( mpn_filter_ptr filter );

mpn_filter_ptr parse_filter( xmlDocPtr doc, xmlNodePtr cur );

#endif
