/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>, Nils Behrens <psi@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

/* 
** $Id: matrixpublish.c,v 1.5 2002/09/12 17:09:13 cvs Exp $ 
*/

#include <time.h>
#include <mpn/matrixpublish.h>
#include "matrix_intern.h"
#include "prelude.h"
#include "sflmime.h"
#include "sflmail.h"
#include "sflsock.h"


//only for debugging...
#include "sflsyst.h"

int send_email( const char* from, const char* to, const char* mailserver, 
		const char* meta, const char* srcfilename )
{
  char empty[5];
  char subject[80];
  char mailername[80];
  int r;
  SMTP smtp;

  strcpy( subject, "Matrix Public Net Broadcast" );
  strcpy( mailername, "libmatrix" );
  strcpy( empty, "" );

  //bzero( &smtp, sizeof( smtp ) );
  memset(&smtp, 0, sizeof(smtp));
  
  
  smtp.strSmtpServer = (char*)mailserver;
  smtp.connect_retry_cnt = 3;
  smtp.retry_wait_time = 0;
  smtp.strSenderUserId = (char*)from;
  smtp.strDestUserIds = (char*)to;
  smtp.strMailerName = mailername;
  smtp.strSubject = subject;
  smtp.strMessageBody = (char*)meta;
  smtp.strBinFiles = (char*)srcfilename;
  smtp.strCcUserIds = empty;
  smtp.strBccUserIds = empty;

  sock_init();
  r = smtp_send_mail_ex( &smtp );  
  sock_term();

  if( r==0 ) return MPN_OK;
  else return MPN_PERM_ERR;
}

int publish_per_email( mpn_identity_ptr id, 
		       mpn_news_ptr newsdoc, 
		       mpn_newsitem_ptr newsitem, 
		       const char* srcfilename )
{
  mpn_person_ptr person;
  mpn_remote_account_ptr account;
  int r;
  char mailreply[20];
  xmlBufferPtr xmlbuffer;
  char text_msg[5000];

  person = mpn_identity_get_person( id );
  account = mpn_identity_get_remote_account( id );

  // Set Mailreply
  if( mpn_remote_account_get_mailreply( account ) ) 
    strcpy( mailreply, "<mailreply/>" );
  else mailreply[0]=0;

  // Dump newsitem
  xmlbuffer = xmlBufferCreate();
  xmlNodeDump( xmlbuffer, newsdoc->doc.doc, newsitem->cur, 1, 1 );

  sprintf( text_msg, "\
<?xml version=\"1.0\"?>\n\
<matrix-publish>\n\
  <username>%s</username>\n\
  %s\n\
  %s\n\
</matrix-publish>", 
	   mpn_remote_account_get_userid( account ), 
	   mailreply, 
	   (char*)xmlBufferContent( xmlbuffer ) );
  
  xmlBufferFree( xmlbuffer );

  // Send EMail
  r = send_email( mpn_remote_account_get_email( account ), 
		  mpn_remote_account_get_serveremail( account ), 
		  mpn_remote_account_get_mailserver( account ), 
		  text_msg, srcfilename );
  
  return r;  
}

int newsitem_addto_local( mpn_identity_ptr id, 
			  mpn_news_ptr newsdoc, 
			  mpn_newsitem_ptr newsitem )
{
  mpn_news_ptr publicdoc;
  mpn_local_account_ptr la;
  int r, i, alreadyhere;
  const char* newid;

  la = mpn_identity_get_local_account( id );
  r = mpn_news_open_foreign( mpn_local_account_get_news( la ),
			     MPN_OPEN_READWRITE, &publicdoc, 
			     "matrix-public-news" );
  if( r!=MPN_OK ) {
    fprintf( stderr, "cannot open local public newsfile %s\n",
	     mpn_local_account_get_news( la ) );
    return MPN_PERM_ERR;
  }

  newid = mpn_sig_get_id( mpn_newsitem_get_sig( newsitem ) );
  alreadyhere = 0;

  for( i=0; i<mpn_news_get_len( publicdoc ); i++ ) {
    if( strcmp( newid, mpn_sig_get_id( mpn_newsitem_get_sig( mpn_news_get_item( publicdoc, i ) ) ) )==0 ) {
      alreadyhere = 1;
    }
  }

  if( !alreadyhere ) {
    mpn_news_add_item( publicdoc, newsitem );
  }

  mpn_news_close( publicdoc );
  return MPN_OK;
}

int publish_per_local( mpn_identity_ptr id, 
		       mpn_news_ptr newsdoc, 
		       mpn_newsitem_ptr newsitem, 
		       const char* srcfilename )
{
  int r;
  FILE* srcfile;
  char filename[MPN_STRING_LEN];
  FILE* file;
  long written, read;
  char* buf[0xFFFF];

  r = newsitem_addto_local( id, newsdoc, newsitem );
  if( r!=MPN_OK ) return r;

  if( srcfilename ) {

    sprintf( filename, "%s/%s",
	     mpn_local_account_get_space( mpn_identity_get_local_account(id) ),
	     mpn_sig_get_id( mpn_newsitem_get_sig( newsitem ) ) );
    
    file = fopen( filename, "w" );
    if( file==NULL ) {
      fprintf(stderr,"error creating file in local space_dir: %s\n",filename );
      return MPN_PERM_ERR;
    }

    srcfile = fopen( srcfilename, "r" );
    if( srcfile==NULL ) {
      fprintf( stderr,"error opening file: %s\n", srcfilename );
      fclose( file );
      return MPN_PERM_ERR;
    }

    read = fread( buf, 1, sizeof( buf ), srcfile );
    while( read ) {
      written = fwrite( buf, 1, read, file );
      if( written!=read ) {
	fprintf(stderr,"error writing file to local space_dir: %s\n",filename);
	fclose( file );
	fclose( srcfile );
	return MPN_PERM_ERR;
      }
      read = fread( buf, 1, sizeof( buf ), srcfile );
    }

    fclose( srcfile );
    fclose( file );

  }
  return MPN_OK;
}

int recommend_per_email( mpn_identity_ptr id, 
			 mpn_news_ptr newsdoc, 
			 mpn_newsitem_ptr newsitem )
{
  mpn_person_ptr person;
  mpn_remote_account_ptr account;
  int r;
  char mailreply[20];
  xmlBufferPtr xmlbuffer;
  char text_msg[5000];

  //printf( "recommend_per_email1\n" );

  person = mpn_identity_get_person( id );
  account = mpn_identity_get_remote_account( id );

  // Set Mailreply
  if( mpn_remote_account_get_mailreply( account ) ) 
    strcpy( mailreply, "<mailreply/>" );
  else mailreply[0]=0;

  // Dump newsitem
  xmlbuffer = xmlBufferCreate();
  xmlNodeDump( xmlbuffer, newsdoc->doc.doc, newsitem->cur, 1, 1 );

  sprintf( text_msg, "\
<?xml version=\"1.0\"?>\n\
<matrix-recommend>\n\
  <username>%s</username>\n\
  %s\n\
  %s\n\
</matrix-recommend>", 
	   mpn_remote_account_get_userid( account ), 
	   mailreply, 
	   (char*)xmlBufferContent( xmlbuffer ) );
  
  xmlBufferFree( xmlbuffer );

  // Send EMail
  r = send_email( mpn_remote_account_get_email( account ), 
		  mpn_remote_account_get_serveremail( account ), 
		  mpn_remote_account_get_mailserver( account ), 
		  text_msg, NULL );

  return r;
}

int recommend_per_local( mpn_identity_ptr id, 
			 mpn_news_ptr newsdoc, 
			 mpn_newsitem_ptr newsitem )
{
  return newsitem_addto_local( id, newsdoc, newsitem );
}

int mpn_publish_newsitem_file( mpn_identity_ptr id, 
			       mpn_news_ptr newsdoc, 
			       mpn_newsitem_ptr newsitem, 
			       const char* srcfilename )
{
  if( mpn_identity_get_local_account( id ) ) {
    return publish_per_local( id, newsdoc, newsitem, srcfilename );
  } else if( mpn_identity_get_remote_account( id ) ) {
    return publish_per_email( id, newsdoc, newsitem, srcfilename );
  } else {
    return MPN_PERM_ERR;
  }
}

int mpn_recommend_newsdoc( mpn_identity_ptr id, mpn_news_ptr newsdoc )
{
  int i, r;

  r = MPN_PERM_ERR;
  for( i=0; i<mpn_news_get_len( newsdoc ); i++ ) {
    r = mpn_recommend_newsitem( id, newsdoc, mpn_news_get_item( newsdoc, i ) );
  }
  return r;
}

int mpn_recommend_newsitem( mpn_identity_ptr id, mpn_news_ptr newsdoc, 
			    mpn_newsitem_ptr newsitem )
{
  //int r;

  if( mpn_identity_get_local_account( id ) ) {
    return recommend_per_local( id, newsdoc, newsitem );
  } else if( mpn_identity_get_remote_account( id ) ) {
    return recommend_per_email( id, newsdoc, newsitem );
  } else {
    return MPN_PERM_ERR;
  }
}

//TODO: this goes into data!!!!!
/*
mpn_news_ptr mail_newsdoc;
const char* attachbuf;
int attachlen;

void mime_callback( GMimePart *part, gpointer data)
{
  const GMimeContentType* type;
  const char* content;
  int len, r;

  type = g_mime_part_get_content_type( part );
  //printf( "mime-typ: %s/%s\n", type->type, type->subtype );

  if( g_mime_content_type_is_type( type, "text", "*" ) ) {

    content = g_mime_part_get_content( part, &len );
    //printf( "%s\n", content );
    r = mpn_news_open_mem( content, &mail_newsdoc );
    if( r!=MPN_OK ) fprintf( stderr, "error parsing mail\n" );

  } else if( g_mime_content_type_is_type( type, "application", "*" ) ) {

    attachbuf = g_mime_part_get_content( part, &attachlen );

  }

}

int mpn_parse_email( FILE* file, mpn_news_ptr* newsdoc_ret, char* from,
		     char** filebuf, long* size )
{
  GMimeMessage* mail;
  const char* f;

  mail_newsdoc = NULL;
  attachbuf = NULL;
  attachlen = 0;

  mail = g_mime_parser_construct_message_from_file( file, 0 );
  
  f = g_mime_message_get_sender( mail );
  strcpy( from, f );

  g_mime_message_foreach_part( mail, mime_callback, NULL );

  *newsdoc_ret =  mail_newsdoc;
  if( attachlen>0 ) {
    *filebuf = malloc( attachlen );
    memcpy( *filebuf, attachbuf, attachlen );
    *size = attachlen;
  } else {
    *filebuf = NULL;
    *size = 0;
  }

  g_mime_message_destroy( mail );
  return MPN_OK;
}
*/
/*
int mpn_parse_email( FILE* file, mpn_news_ptr* newsdoc_ret, char* from,
		     char** filebuf, long* size )
{
  char line[MPN_STRING_LEN];

  while( !eof( file ) ) {
    fread( line, 1, MPN_STRING_LEN, file );
    
}
*/
