/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <mpn/matrix.h>
#include <stdio.h>
#include <string.h>

void add_friend( const char* name, const char* home, int bot, 
		 const char* cookie )
{
  mpn_prefs_ptr prefsdoc;
  mpn_friend_ptr friend;
  mpn_person_ptr person;
  int r, alreadyhere, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  alreadyhere = 0;
  for( i=0; i<mpn_prefs_get_friend_count( prefsdoc ); i++ ) {
    friend = mpn_prefs_get_friend( prefsdoc, i );
    person = mpn_friend_get_person( friend );
    if( strcmp( mpn_person_get_home( person ), home )==0 ) {
      alreadyhere = 1;
    }
  }

  if( !alreadyhere ) {
    mpn_prefs_add_friend( prefsdoc, name, home, 1, bot, cookie );
  } else {
    printf( "  Friend is already in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

void del_friend( const char* home )
{
  mpn_prefs_ptr prefsdoc;
  mpn_friend_ptr friend;
  mpn_person_ptr person;
  int r, fi, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  fi = -1;
  for( i=0; i<mpn_prefs_get_friend_count( prefsdoc ); i++ ) {
    friend = mpn_prefs_get_friend( prefsdoc, i );
    person = mpn_friend_get_person( friend );
    if( strcmp( mpn_person_get_home( person ), home )==0 ) {
      fi = i;
    }
  }

  if( fi>-1 ) {
    mpn_prefs_del_friend( prefsdoc, fi );
  } else {
    printf( "  Friend not found in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

void add_identity( const char* name, const char* home, const char* userid,
		   const char* email, const char* server_email,
		   const char* public_space )
{
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr id;
  mpn_person_ptr person;
  int r, alreadyhere, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  alreadyhere = 0;
  for( i=0; i<mpn_prefs_get_identity_count( prefsdoc ); i++ ) {
    id = mpn_prefs_get_identity( prefsdoc, i );
    person = mpn_identity_get_person( id );
    if( strcmp( mpn_person_get_home( person ), home )==0 ) {
      alreadyhere = 1;
    }
  }

  if( !alreadyhere ) {
    mpn_prefs_add_remote_identity( prefsdoc, name, home, userid, email,
				   server_email, public_space, "localhost" );
  } else {
    printf( "  Identity is already in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

void del_identity( const char* home )
{
  mpn_prefs_ptr prefsdoc;
  mpn_identity_ptr id;
  mpn_person_ptr person;
  int r, fi, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  fi = -1;
  for( i=0; i<mpn_prefs_get_identity_count( prefsdoc ); i++ ) {
    id = mpn_prefs_get_identity( prefsdoc, i );
    person = mpn_identity_get_person( id );
    if( strcmp( mpn_person_get_home( person ), home )==0 ) {
      fi = i;
    }
  }

  if( fi>-1 ) {
    mpn_prefs_del_identity( prefsdoc, fi );
  } else {
    printf( "  Identity not found in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

void add_idnamelist( const char* list, int id, const char* name )
{
  mpn_prefs_ptr prefsdoc;
  mpn_id_name_list_ptr idnamelist;
  mpn_id_name_ptr idname;
  int r, alreadyhere, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  idnamelist = mpn_prefs_get_idnamelist( prefsdoc, list );
  if( !idnamelist ) {
    idnamelist = mpn_prefs_add_idnamelist( prefsdoc, list );
  }

  alreadyhere = 0;
  for( i=0; i<mpn_idnamelist_get_len( idnamelist ); i++ ) {
    idname = mpn_idnamelist_get_idname( idnamelist, i );
    if( mpn_idname_get_id( idname ) == id ) {
      alreadyhere = 1;
    }
  }

  if( !alreadyhere ) {
    mpn_idnamelist_add_entry( idnamelist, id, name );
  } else {
    printf( "  Entry is already in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

void del_idnamelist( const char* list, int id )
{
  mpn_prefs_ptr prefsdoc;
  mpn_id_name_list_ptr idnamelist;
  mpn_id_name_ptr idname;
  int r, fi, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  idnamelist = mpn_prefs_get_idnamelist( prefsdoc, list );
  if( !idnamelist ) {
    return;
  }

  fi = -1;
  for( i=0; i<mpn_idnamelist_get_len( idnamelist ); i++ ) {
    idname = mpn_idnamelist_get_idname( idnamelist, i );
    if( mpn_idname_get_id( idname ) == id ) {
      fi = i;
    }
  }

  if( fi>-1 ) {
    mpn_idnamelist_del_entry( idnamelist, fi );
  } else {
    printf( "  Entry not found in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

void default_idnamelist( const char* list, int id )
{
  mpn_prefs_ptr prefsdoc;
  mpn_id_name_list_ptr idnamelist;
  mpn_id_name_ptr idname;
  int r, fi, i;

  r = mpn_prefs_open_private( MPN_OPEN_READWRITE, &prefsdoc );
  if( r!=MPN_OK ) return;
  
  idnamelist = mpn_prefs_get_idnamelist( prefsdoc, list );
  if( !idnamelist ) {
    return;
  }

  fi = -1;
  for( i=0; i<mpn_idnamelist_get_len( idnamelist ); i++ ) {
    idname = mpn_idnamelist_get_idname( idnamelist, i );
    if( mpn_idname_get_id( idname ) == id ) {
      fi = i;
    }
  }

  if( fi>-1 ) {
    mpn_idnamelist_set_default( prefsdoc, idnamelist, fi );
  } else {
    printf( "  Entry not found in your prefs.xml\n" );
  }

  mpn_prefs_close( prefsdoc );
}

int main( int argc, char** argv )
{
  char buf[MPN_STRING_LEN];
  char* bufref;
  char* proto;
  char* cmd;
  char* name;
  char* home;
  char* userid;
  char* email;
  char* server_email;
  char* public_space;
  char* list;
  char* idstr;
  char* cookie;
  int id;

  mpn_init();

  if( argc<2 ) return -1;
  
  strcpy( buf, argv[1] );
  bufref = buf;

  proto = strtok( bufref, "|" );
  if( !proto ) return -1;
  if( strcmp( proto, "matrix://" ) !=0 ) return -1;

  cmd = strtok( NULL, "|" );
  if( !cmd ) return -1;

  if( strcmp( cmd, "addfriend" )==0 ) {
    name = strtok( NULL, "|" );
    if( !name ) return -1;
    home = strtok( NULL, "|" );
    if( !home ) return -1;
    cookie = strtok( NULL, "|" );

    printf( "matrixurl\n  Adding friend: %s, %s\n", name, home );
    add_friend( name, home, 0, cookie );

  } else if( strcmp( cmd, "addbot" )==0 ) {
    name = strtok( NULL, "|" );
    if( !name ) return -1;
    home = strtok( NULL, "|" );
    if( !home ) return -1;
    cookie = strtok( NULL, "|" );
    
    printf( "matrixurl\n  Adding bot: %s, %s\n", name, home );
    add_friend( name, home, 1, cookie );

  } else if( strcmp( cmd, "delfriend" )==0 ) {
    home = strtok( NULL, "|" );
    if( !home ) return -1;
    
    printf( "matrixurl\n  Deleting friend: %s\n", home );
    del_friend( home );

  } else if( strcmp( cmd, "addidentity" )==0 ) {
    name = strtok( NULL, "|" );
    if( !name ) return -1;
    home = strtok( NULL, "|" );
    if( !home ) return -1;
    userid = strtok( NULL, "|" );
    if( !userid ) return -1;
    email = strtok( NULL, "|" );
    if( !email ) return -1;
    server_email = strtok( NULL, "|" );
    if( !server_email ) return -1;
    public_space = strtok( NULL, "|" );
    if( !public_space ) return -1;
    
    printf( "matrixurl\n  Adding identity: %s, %s\n", name, home );
    add_identity( name, home, userid, email, server_email, public_space );

  } else if( strcmp( cmd, "delidentity" )==0 ) {
    home = strtok( NULL, "|" );
    if( !home ) return -1;
    
    printf( "matrixurl\n  Deleting identity: %s\n", home );
    del_identity( home );

  } else if( strcmp( cmd, "addidname" )==0 ) {
    list = strtok( NULL, "|" );
    if( !list ) return -1;
    idstr = strtok( NULL, "|" );
    if( !idstr ) return -1;
    id = atoi( idstr );
    name = strtok( NULL, "|" );
    if( !name ) return -1;
    
    printf( "matrixurl\n  Adding to list %s: %d, %s\n", list, id, name );
    add_idnamelist( list, id, name );

  } else if( strcmp( cmd, "delidname" )==0 ) {
    list = strtok( NULL, "|" );
    if( !list ) return -1;
    idstr = strtok( NULL, "|" );
    if( !idstr ) return -1;
    id = atoi( idstr );
    
    printf( "matrixurl\n  Deleting from list %s: %d\n", list, id );
    del_idnamelist( list, id );

  } else if( strcmp( cmd, "defaultidname" )==0 ) {
    list = strtok( NULL, "|" );
    if( !list ) return -1;
    idstr = strtok( NULL, "|" );
    if( !idstr ) return -1;
    id = atoi( idstr );
    
    printf( "matrixurl\n  Setting default entry in list %s: %d\n", list, id );
    default_idnamelist( list, id );

  } else {
    return -1;
  }

  mpn_cleanup();
  return 0;
}




