<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xsl:version="1.0">

  <xsl:output method="html"/>
  

  <!-- MAIN -->
  <xsl:template match="/">
    <html>
      <head>
        <title>MPN: prefs.xml</title>
      </head>
      <body>
        <h1>MPN: prefs.xml</h1>

        <hr/>
        <a href="prefs.xml">prefs</a> | 
        <a href="published.xml">published</a> |
        <a href="news.xml">news</a> ||
        <a href="http://matrix.wearlab.de">matrix.wearlab.de</a>
        <hr/>

        <h2>Identities</h2>
        <xsl:if test="not(/*/identity)">
          The first thing you need is at least one identity for MPN.
          You can <a href="http://matrix.wearlab.de/registration.php">register
	  for free</a> at <a href="http://matrix.wearlab.de">
          matrix.wearlab.de</a> or at any other MPN server there might be.
          If your computer has a permanent internet connection and a web
	  server running you can also create an identity on that machine.
          <p/>
        </xsl:if>
        <table border="0" width="100%">
          <xsl:for-each select="/*/identity">
            <tr>
              <td>
                Identity<br/>
                <a>
                  <xsl:attribute name="href">
                    matrix://|delidentity|<xsl:value-of select="person/home"/>
                  </xsl:attribute> 
                  delete
                </a>
              </td>
              <td><table border="5" width="100%">
                <xsl:apply-templates/>
              </table></td>
            </tr>
          </xsl:for-each>
        </table>

        <h2>Friends</h2>
        <xsl:if test="not(/*/friend)">
          MPN does not connect you to other people
	  automatically. You need to specify some friends or persons 
          you trust in. So ask your friends for their identity on MPN
          or look into the <a href="http://matrix.wearlab.de/citizenbook.php">
          Book of Citizens</a> on the <a href="http://matrix.wearlab.de">
          matrix.wearlab.de</a> server.
        </xsl:if>
        <xsl:if test="/*/friend">
          <table border="0" width="100%">
            <xsl:for-each select="/*/friend">
              <tr>
                <td>
                  Friend<br/>
                  <a>
                    <xsl:attribute name="href">
                      matrix://|delfriend|<xsl:value-of select="person/home"/>
                    </xsl:attribute> 
                    kick
                  </a>
                </td>
                <td><table border="5" width="100%">
                  <xsl:apply-templates/>
                </table></td>
              </tr>
            </xsl:for-each>
          </table>
          <p/>
          Look into the <a href="http://matrix.wearlab.de/citizenbook.php">
	  Book of Citizens</a> on the <a href="http://matrix.wearlab.de">
          matrix.wearlab.de</a> server for more friends.
        </xsl:if>

        <h2>ID-Name Lists</h2>
        <table border="0" width="100%">
          <xsl:for-each select="/*/id_name_list">
            <tr>
              <td>
                <xsl:value-of select="@name"/><br/>
                <xsl:if test="@name='category'">
                  <a href="http://matrix.wearlab.de/categories.php">
                    get more</a>
                </xsl:if>
                <xsl:if test="@name='priority'">
                  <a href="http://matrix.wearlab.de/priorities.html">
                    get more</a>
                </xsl:if>
                <xsl:if test="@name='language'">
                  <a href="http://matrix.wearlab.de/languages.html">
                    get more</a>
                </xsl:if>

              </td>
              <td><table border="5" width="100%">
                <xsl:apply-templates/>
              </table></td>
            </tr>
          </xsl:for-each>
        </table>    

        <h2>Application: xmms-matrixnews</h2>
        <table border="0" width="100%">
          <xsl:for-each select="/*/app/xmms-matrixnews/newsblock">
            <tr>
              <td>Newsblock</td>
              <td><table border="5" width="100%">
                <xsl:apply-templates/>
              </table></td>
            </tr>
          </xsl:for-each>
        </table>    

        <h2>Global Filter</h2>
        <table border="5" width="100%">
          <xsl:for-each select="/*/filter">
	    <xsl:apply-templates/>
          </xsl:for-each>
        </table>

        <h2>Default Friend Filter</h2>
        <table border="5" width="100%">
          <xsl:for-each select="/*/default_friendfilter">
	    <xsl:apply-templates/>
          </xsl:for-each>
        </table>

        <h2>Default Bot Filter</h2>
        <table border="5" width="100%">
          <xsl:for-each select="/*/default_botfilter">
	    <xsl:apply-templates/>
          </xsl:for-each>
        </table>

      </body>
    </html>
  </xsl:template>


  <!-- IDENTITIES -->
  <xsl:template match="person">
    <tr>
      <td>Person</td>
      <td><table border="0">
        <xsl:apply-templates/>
      </table></td>
    </tr>
  </xsl:template>
  <xsl:template match="local">
    <tr>
      <td>Local Account</td>
      <td><table border="0">
        <xsl:apply-templates/>
      </table></td>
    </tr>
  </xsl:template>
  <xsl:template match="remote">
    <tr>
      <td>Remote Account</td>
      <td><table border="0">
        <xsl:apply-templates/>
      </table></td>
    </tr>
  </xsl:template>

  <xsl:template match="person/name">
    <tr><td>Name: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="person/home">
    <tr><td>Home: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="userid">
    <tr><td>UserID: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="email">
    <tr><td>Your email address: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="mailserver">
    <tr><td>Use mailserver: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="server_email">
    <tr><td>Server email address: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="public_space">
    <tr><td>Your web space</td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="news_file">
    <tr><td>Newsfile: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="space_dir">
    <tr><td>Web dir: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
        

  <!-- FRIENDS -->
  <xsl:template match="poll">
    <tr><td>Poll </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="trust">
    <tr><td>Trust </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="cookie">
  </xsl:template>


  <!-- FILTER -->
  <xsl:template match="filter">
    <tr>
      <td>Filter:</td>
      <td><table border="5">
        <xsl:apply-templates/>
      </table></td>
    </tr>
  </xsl:template>
  <xsl:template match="xslt_proc">
    <tr><td>XSL Stylesheet: </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>


  <!-- APP xmms-matrixnews -->
  <xsl:template match="xmms-matrixnews">
    <h3>Application: xmms-matrixnews</h3>
    <table border="5">
      <xsl:apply-templates/>
    </table>    
  </xsl:template>

  <xsl:template match="newsblock">
    <tr>
      <td>Newsblock</td>
      <td><table border="0">
        <xsl:apply-templates/>
      </table></td>
    </tr>
  </xsl:template>

  <xsl:template match="prolog">
    <tr><td>Prolog (OGG-file): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="separator">
    <tr><td>Separator (OGG-file): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="epilog">
    <tr><td>Epilog (OGG-file): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="frequency">
    <tr><td>Frequency (per hour): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="offset">
    <tr><td>Offset (in minutes): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="preload">
    <tr><td>Preload News (in minutes): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>
  <xsl:template match="duration">
    <tr><td>Duration (in minutes): </td><td><xsl:apply-templates/></td></tr>
  </xsl:template>


  <!-- ID_NAME_LISTS -->
  <xsl:template match="entry">
    <tr><xsl:apply-templates/></tr>
  </xsl:template>
  <xsl:template match="entry/id">
    <td>
      <xsl:apply-templates/>
      <xsl:if test="../default"> (default)</xsl:if>
    </td>
  </xsl:template>
  <xsl:template match="entry/name">
    <td>
      <xsl:apply-templates/>
    </td>
    <td>
      <a>
        <xsl:attribute name="href">
          matrix://|delidname|<xsl:value-of select="../../@name"/>|<xsl:value-of select="../id"/>
        </xsl:attribute>
        delete
      </a> 
      <xsl:if test="not(../default)"> | <a>
          <xsl:attribute name="href">
            matrix://|defaultidname|<xsl:value-of select="../../@name"/>|<xsl:value-of select="../id"/>
          </xsl:attribute>
          default
        </a>
      </xsl:if>
    </td>
  </xsl:template>

</xsl:stylesheet>
