/*
** Copyright (C) 2002 Tom Nicolai <nicolai@tzi.de>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <mpn/matrix.h>
#include <time.h>

void play_music()
{
  mpn_news_ptr newsdoc;
  mpn_newsitem_ptr item;
  mpn_app_ptr app;
  mpn_app_block_ptr djblock, musicblock;
  mpn_sig_ptr sig;
  mpn_file_ptr file;
  mpn_docinfo_ptr docinfo;
  int j, r;
  time_t timestamp;
  char musicurl[MPN_STRING_LEN];

  r = mpn_news_open_private( MPN_OPEN_READWRITE, &newsdoc );
  if( r!=MPN_OK ) {
    fprintf( stderr, "error opening news file\n" );
  }

  // play
  if( r==MPN_OK ) {
    docinfo = mpn_news_get_docinfo( newsdoc );
	
    for( j=0; j<mpn_news_get_len( newsdoc ); j++ ) {
      item = mpn_news_get_item( newsdoc, j );

      app = mpn_newsitem_get_app( item );
      djblock = mpn_app_get_block( app, "dj" );
      if( djblock ) {

	musicblock = mpn_app_get_block( app, "xmms-matrixmusic" );
	if( !musicblock ) {

	  sig = mpn_newsitem_get_sig( item );
	  file = mpn_sig_get_file( sig );
	  if( file ) {

	    if( mpn_file_get_url( file ) ) {

	      sprintf( musicurl, "xmms -e \"%s\"", mpn_file_get_url( file ) );
	      printf( "%s\n", musicurl );
	      system( musicurl );
	      
	      musicblock = mpn_app_add_block( app, "xmms-matrixmusic" );
	      timestamp = time( NULL );
	      mpn_app_block_set_long( docinfo, musicblock, "played_sec", 
				      timestamp );
	      mpn_app_block_set_string( docinfo, musicblock, "played_txt", 
					ctime( &timestamp ));
	    }
	  }
	}
      }
    }
    mpn_news_close( newsdoc );
  }
}

int main()
{
  mpn_init();

  while( 1 ) {
    printf( "checking for music\n" );
    play_music();
    sleep( 60 );
  }

  mpn_cleanup();
  return 0;
}
